from six import iteritems
import pyutilib
import coopr
from coopr.opt import SolverFactory

cplex_available = False
cplexamp_available = False
cplexpy_available = False
try:
    cplex = coopr.plugins.solvers.CPLEX(keepfiles=True)
    cplex_available = (not cplex.executable() is None) and cplex.available(False)
    asl = coopr.plugins.solvers.ASL(keepfiles=True, options={'solver':'cplexamp'})
    cplexamp_available = cplex_available and (not asl.executable() is None) and asl.available(False)
    try:
        import cplex
        cplexpy_available = True
    except ImportError:
        cplexpy_available = False
except pyutilib.common.ApplicationError:
    cplexamp_available=False

gurobi_available = False
gurobi_ampl_available = False
gurobipy_available = False
try:
    gurobi = coopr.plugins.solvers.GUROBI(keepfiles=True)
    gurobi_available = (not gurobi.executable() is None) and gurobi.available(False)
    asl = coopr.plugins.solvers.ASL(keepfiles=True, options={'solver':'gurobi_ampl'})
    gurobi_ampl_available = gurobi_available and (not asl.executable() is None) and asl.available(False)
    try:
        import gurobipy
        gurobipy_available = True
    except ImportError:
        gurobipy_available = False
except pyutilib.common.ApplicationError:
    gurobi_ampl_available=False

glpk_available = False
glpkpy_available = False
glpk_old = False
try:
    glpk = coopr.plugins.solvers.GLPK(keepfiles=True)
    glpk_available = (not glpk.executable() is None) and glpk.available(False)
    if 'GLPKSHELL_old' in str(glpk.__class__):
        glpk_old = True
    try:
        import glpk
        glpkpy_available = True
    except ImportError:
        glpkpy_available = False
except pyutilib.common.ApplicationError:
    pass

cbc_available = False
cbc_ampl_available = False
try:
    cbc = coopr.plugins.solvers.CBC(keepfiles=True)
    cbc_available = (not cbc.executable() is None) and cbc.available(False)
    cbc_ampl = coopr.plugins.solvers.CBC(keepfiles=True,solver_io='nl')
    cbc_ampl_available = (not cbc_ampl is None) and (not cbc_ampl.executable() is None) and cbc_ampl.available(False)
except pyutilib.common.ApplicationError:
    pass

pico_available = False
pico_ampl_available = False
try:
    pico = coopr.plugins.solvers.PICO(keepfiles=True)
    pico_available = (not pico.executable() is None) and pico.available(False)
    pico_ampl = coopr.plugins.solvers.PICO(keepfiles=True,solver_io='nl')
    pico_ampl_available = (not pico_ampl is None) and (not pico_ampl.executable() is None) and pico_ampl.available(False)
except pyutilib.common.ApplicationError:
    pass

xpress_available = False
try:
    xpress = coopr.plugins.solvers.XPRESS(keepfiles=True)
    xpress_available = (not xpress.executable() is None) and xpress.available(False)
except pyutilib.common.ApplicationError:
    pass

ipopt_available = False
try:
    asl = coopr.plugins.solvers.ASL(keepfiles=True, options={'solver':'ipopt'})
    ipopt_available = (not asl.executable() is None) and asl.available(False)
except pyutilib.common.ApplicationError:
    ipopt_available=False

scip_available = False
try:
    scip = coopr.plugins.solvers.SCIPAMPL(keepfiles=True)
    scip_available = (not scip.executable() is None) and scip.available(False)
except pyutilib.common.ApplicationError:
    scip_available = False

class SolverTestCase(object):

    def __init__(self,name=None,io=None,available=False,**kwds):
        assert (name is not None) and (type(name) is str)
        assert (io is not None) and (type(io) is str)
        assert (available in [True,False])
        self.name = name
        self.io = io
        self.available = available 
        self.capabilities = kwds.pop('capabilities',[])
        self.export_suffixes = kwds.pop('export_suffixes',[])
        self.import_suffixes = kwds.pop('import_suffixes',[])
        self.options = kwds.pop('options',{})
        assert type(self.capabilities) in [list,tuple]
        assert type(self.export_suffixes) in [list,tuple]
        assert type(self.import_suffixes) in [list,tuple]
        assert type(self.options) is dict
        for tag in self.capabilities:
            assert type(tag) is str
        for tag in self.export_suffixes:
            assert type(tag) is str
        for tag in self.import_suffixes:
            assert type(tag) is str
        self.solver = None

    def initialize(self):
        self.solver = None
        opt = SolverFactory(self.name,solver_io=self.io)
        if opt is not None:
            for key,value in iteritems(self.options):
                opt.options[key] = value
        self.solver = opt

    def has_capability(self,tag):
        if tag in self.capabilities:
            return True
        return False

    def __str__(self):
        tmp  = "SolverTestCase:\n"
        tmp += "\tname = "+self.name+"\n"
        tmp += "\tio = "+self.io+"\n"
        tmp += "\tavailable = "+str(self.available)+"\n"
        if self.solver is not None:
            tmp += "\tversion = "+str(self.solver.version())+"\n"
        else:
            tmp += "\tversion = unknown\n"
        tmp += "\tcapabilities: "
        if len(self.capabilities):
            tmp += "\n"
            for tag in self.capabilities:
                tmp += "\t   - "+tag+"\n"
        else:
            tmp += "None\n"
        tmp += "\tsuffixes: \n"
        tmp += "\t  export: "
        if len(self.export_suffixes):
            tmp += "\n"
            for tag in self.export_suffixes:
                tmp += "\t   - "+tag+"\n"
        else:
            tmp += "None\n"
        tmp += "\t  import: "
        if len(self.import_suffixes):
            tmp += "\n"
            for tag in self.import_suffixes:
                tmp += "\t   - "+tag+"\n"
        else:
            tmp += "None\n"
        return tmp
        

# The capabilities listed below should be what is
# advertised by the solver and not the Pyomo plugin
# But we are testing that they match
testCases = []

#
#    ADD CPLEX TEST CASES
#
cplex_capabilities = ['linear',
                      'integer',
                      'quadratic_objective',
                      'quadratic_constraint',
                      'sos1',
                      'sos2']
testCases.append( SolverTestCase(name='cplex',
                                 io='lp',
                                 capabilities=cplex_capabilities,
                                 import_suffixes=['slack'], #dual, rc
                                 available=cplex_available) )
testCases.append( SolverTestCase(name='cplexamp',
                                 io='nl',
                                 capabilities=cplex_capabilities,
                                 import_suffixes=[], #dual
                                 available=cplexamp_available) )
testCases.append( SolverTestCase(name='cplex',
                                 io='python',
                                 capabilities=cplex_capabilities,
                                 import_suffixes=['slack'], #dual, rc
                                 available=cplexpy_available) )


#
#    ADD GUROBI TEST CASES
#
gurobi_capabilities = ['linear',
                       'integer',
                       'quadratic_objective',
                       'quadratic_constraint',
                       'sos1',
                       'sos2']
# **NOTE: Gurobi does not handle quadratic constraints before
#         Major Version 5
testCases.append( SolverTestCase(name='gurobi',
                                 io='lp',
                                 capabilities=gurobi_capabilities,
                                 import_suffixes=['rc','dual','slack'],
                                 available=gurobi_available) )
testCases.append( SolverTestCase(name='gurobi_ampl',
                                 io='nl',
                                 capabilities=gurobi_capabilities,
                                 import_suffixes=['dual'],
                                 available=gurobi_ampl_available,
                                 options={'qcpdual':1,'simplex':1}) )
testCases.append( SolverTestCase(name='gurobi',
                                 io='python',
                                 capabilities=gurobi_capabilities,
                                 import_suffixes=['rc','dual','slack'],
                                 available=gurobipy_available) )

#
#    ADD GLPK TEST CASES
#
glpk_capabilities = ['linear',
                     'integer']
if glpk_old is True:
    glpk_import_suffixes = ['dual']
else:
    glpk_import_suffixes = ['rc','dual']
testCases.append( SolverTestCase(name='glpk',
                                 io='lp',
                                 capabilities=glpk_capabilities,
                                 import_suffixes=glpk_import_suffixes,
                                 available=glpk_available) )
testCases.append( SolverTestCase(name='glpk',
                                 io='python',
                                 capabilities=glpk_capabilities,
                                 import_suffixes=[],
                                 available=glpkpy_available) )


#
#    ADD CBC TEST CASES
#
cbc_capabilities = ['linear',
                    'integer']
testCases.append( SolverTestCase(name='cbc',
                                 io='lp',
                                 capabilities=cbc_capabilities,
                                 import_suffixes=['rc','dual'],
                                 available=cbc_available) )
testCases.append( SolverTestCase(name='cbc',
                                 io='nl',
                                 capabilities=cbc_capabilities,
                                 import_suffixes=['dual'],
                                 available=cbc_ampl_available) )

#
#    ADD PICO TEST CASES
#
pico_capabilities = ['linear',
                     'integer']
testCases.append( SolverTestCase(name='pico',
                                 io='lp',
                                 capabilities=pico_capabilities,
                                 import_suffixes=['dual'],
                                 available=pico_available) )
testCases.append( SolverTestCase(name='pico',
                                 io='nl',
                                 capabilities=pico_capabilities,
                                 import_suffixes=['dual'],
                                 available=pico_ampl_available) )

#
#    ADD XPRESS TEST CASES
#
xpress_capabilities = ['linear',
                     'integer',
                     'quadratic_objective',
                     'quadratic_constraint',
                     'sos1',
                     'sos2']
testCases.append( SolverTestCase(name='xpress',
                                 io='lp',
                                 capabilities=xpress_capabilities,
                                 import_suffixes=['rc','dual','slack'],
                                 available=xpress_available) )

#
#    ADD IPOPT TEST CASES
#
ipopt_capabilities = ['linear',
                      'quadratic_objective',
                      'quadratic_constraint']
testCases.append( SolverTestCase(name='ipopt',
                                 io='nl',
                                 capabilities=ipopt_capabilities,
                                 import_suffixes=['dual'],
                                 available=ipopt_available) )

#
#    ADD SCIP TEST CASES
#
scip_capabilities = ['linear',
                         'integer',
                         'quadratic_objective',
                         'quadratic_constraint',
#                         'sos1', # scip does not handle these currently, I've reported this bug to SCIP
                         'sos2']
testCases.append( SolverTestCase(name='scip',
                                 io='nl',
                                 capabilities=scip_capabilities,
                                 import_suffixes=[],
                                 available=scip_available) )


if __name__ == "__main__":

    for case in testCases:
        case.initialize()
        print(case)



from coopr.pyomo import *

INDEX_SET = ['a','b']
PIECEWISE_PTS = {'a':[1,2,3], 'b':[1,2,3]}
F = lambda x: x**2

def define_model():
    model = ConcreteModel()

    def SOS_indices_init(model,t):
        return [(t,i) for i in xrange(1,len(PIECEWISE_PTS[t]))]
    def ub_indices_init(model):
        return [(t,i) for t in INDEX_SET for i in xrange(1,len(PIECEWISE_PTS[t]))]
    
    model.SOS_indices = Set(INDEX_SET,dimen=2, ordered=True, initialize=SOS_indices_init)
    model.ub_indices = Set(ordered=True, dimen=2,initialize=ub_indices_init)

    model.x = Var(INDEX_SET)
    model.Fx = Var(INDEX_SET)
    #Add SOS1 variable to model
    model.y = Var(model.ub_indices,within=NonNegativeReals)

    def constraint1_rule(model,t,i):
        return model.Fx[t] - (F(PIECEWISE_PTS[t][i-1]) + ((F(PIECEWISE_PTS[t][i])-F(PIECEWISE_PTS[t][i-1]))/(PIECEWISE_PTS[t][i]-PIECEWISE_PTS[t][i-1]))*(model.x[t]-PIECEWISE_PTS[t][i-1])) <= 25.0*(1-model.y[t,i])
    def constraint2_rule(model,t):
        return sum(model.y[t,j] for j in xrange(1,len(PIECEWISE_PTS[t]))) == 1

    model.obj = Objective(expr=summation(model.Fx), sense=maximize)
    model.constraint1 = Constraint(model.ub_indices,rule=constraint1_rule)
    model.constraint2 = Constraint(INDEX_SET,rule=constraint2_rule)
    model.SOS_set_constraint = SOSConstraint(INDEX_SET, var=model.y, set=model.SOS_indices, sos=1)

    #Fix the answer for testing purposes
    model.set_answer_constraint2 = Constraint(expr= model.x['a'] == 1.5)
    model.set_answer_constraint1 = Constraint(expr= model.x['b'] == 2.5)

    return model

model = define_model()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['display']

import logging
import sys

logger = logging.getLogger('coopr.pyomo')


def display(obj, ostream=None):
    """ Display data in a Pyomo object"""
    if ostream is None:
        ostream = sys.stdout
    try:
        display_fcn = obj.display
    except AttributeError:
        raise TypeError(
            "Error trying to display values for object of type %s:\n"
            "\tObject does not support the 'display()' method"
            % (type(obj), ) )
    try:
        display_fcn(ostream=ostream)
    except Exception:
        err = sys.exc_info()[1]
        logger.error(
            "Error trying to display values for object of type %s:\n\t%s"
            % (type(obj), err) )
        raise 


def create_name(name, ndx):
    if ndx is None:
        return name
    if type(ndx) is tuple:
        tmp = str(ndx).replace(', ',',')
        return name+"["+tmp[1:-1]+"]"
    return name+"["+str(ndx)+"]"


def apply_indexed_rule(obj, rule, model, index, options=None):
    try:
        if options is None:
            if index.__class__ is tuple:
                return rule(model, *index)
            return rule(model, index)
        else:
            if index.__class__ is tuple:
                return rule(model, *index, **options)
            return rule(model, index, **options)
    except TypeError:
        try:
            if options is None:
                return rule(model)
            else:
                return rule(model, **options)
        except:
            # Nothing appears to have matched... re-trigger the original
            # TypeError
            if options is None:
                if index.__class__ is tuple:
                    return rule(model, *index)
                return rule(model, index)
            else:
                if index.__class__ is tuple:
                    return rule(model, *index, **options)
                return rule(model, index, **options)

def apply_parameterized_indexed_rule(obj, rule, model, param, index):
    if index.__class__ is tuple:
        return rule(model, param, *index)
    return rule(model, param, index)




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['Param']

import sys
import types
import logging
import weakref
import six
from six import iteritems, iterkeys

import pyutilib.math

from coopr.pyomo.base.component import Component, ComponentData, register_component
from coopr.pyomo.base.sparse_indexed_component import SparseIndexedComponent
from coopr.pyomo.base.indexed_component import IndexedComponent
from coopr.pyomo.base.misc import apply_indexed_rule, apply_parameterized_indexed_rule, \
     create_name
from coopr.pyomo.base.numvalue import NumericConstant, NumericValue, native_types, value
from coopr.pyomo.base.set_types import Any, Reals

logger = logging.getLogger('coopr.pyomo')


class _ParamData(ComponentData, NumericValue):
    """Holds the numeric value of a mutable parameter"""

    __slots__ = ('domain', 'value')

    def __init__(self, owner, domain, value):
        """Constructor"""
        ComponentData.__init__(self, owner)
        if domain is None:
            self.domain = Reals
        else:
            self.domain = domain
        if pyutilib.math.is_nan(value):
            self.value = pyutilib.math.nan
        else:
            self.value = value

    def __getstate__(self):
        state = super(_ParamData, self).__getstate__()
        for i in _ParamData.__slots__:
            state[i] = getattr(self, i)
        return state

    # Note: because NONE of the slots on this class need to be edited,
    # we don't need to implement a specialized __setstate__ method, and
    # can quietly rely on the super() class's implementation.
    # def __setstate__(self, state):
    #     pass

    def is_constant(self):
        return False

    def is_fixed(self):
        return True

    def clear(self):
        self.value = None

    def __nonzero__(self):
        """Return True if the value is defined and non-zero."""
        if self.value:
            return True
        if self.value is None:
            raise ValueError("Numeric Constant: value is undefined")
        return False

    __bool__ = __nonzero__

    def __call__(self, exception=True):
        """Return the value of this object."""
        return self.value

    def pprint(self, ostream=None, verbose=False):
        if ostream is None:         #pragma:nocover
            ostream = sys.stdout
        print >>ostream, str(self),


class Param(SparseIndexedComponent):
    """A parameter value, which may be defined over a index"""

    """ Constructor
        Arguments:
           name        The name of this parameter
           index       The index set that defines the distinct parameters.
                         By default, this is None, indicating that there
                         is a single parameter.
           within      A set that defines the type of values that
                         each parameter must be.
           validate    A rule for validating this parameter w.r.t. data
                         that exists in the model
           default     A scalar, rule, or dictionary that defines default 
                         values for this parameter
           initialize  A dictionary or rule for setting up this parameter 
                         with existing model data
           nochecking  If true, various checks regarding valid domain and index
                         values are skipped. Only intended for use by algorithm
                         developers - not modelers!
    """

    DefaultMutable = False

    def __new__(cls, *args, **kwds):
        if cls != Param:
            return super(Param, cls).__new__(cls)
        if args == ():
            return _ParamElement.__new__(_ParamElement)
        else:
            return _ParamArray.__new__(_ParamArray)

    def __init__(self, *args, **kwd):
        self._rule       = kwd.pop('initialize', None )
        self._validate   = kwd.pop('validate', None )
        self.domain      = kwd.pop('within', Any )
        self.nochecking  = kwd.pop('nochecking',False)
        self._mutable    = kwd.pop('mutable', Param.DefaultMutable )

        if 'repn' in kwd:
            logger.warning(
                "DEPRECATED: The use of the 'repn' keyword when constructing "
                "Params has been deprecated and will be removed in Coopr 4.0.")
            kwd.pop('repn')
        if 'rule' in kwd:
            logger.warning(
                "DEPRECATED: The use of the 'rule' keyword when constructing "
                "Params has been deprecated and will be removed in Coopr 4.0."
                "\n\tPlease use 'initialize'")
            if self._rule is not None:
                raise ValueError(
                    "Cannot specify both initialize and rule parameters when "
                    "constructing a Param object")
            self._rule = self._rule = kwd.pop('rule')

        # Use set_default() so that we vaidate the value against the domain
        self._default_val = None
        default = kwd.pop('default', None )
        
        #
        kwd.setdefault('ctype', Param)
        SparseIndexedComponent.__init__(self, *args, **kwd)

        self.set_default(default)

        # Because we want to defer the check for defined values until
        # runtime, we will undo the weakref to ourselves
        if not self.is_indexed():
            del self._data[None]

    def pprint(self, ostream=None, verbose=False):
        if ostream is None:
            ostream = sys.stdout
        ostream.write("   %s : " % (self.name,))
        if self.doc is not None:
            ostream.write(" %s\n   " % (self.doc,))
        ostream.write("\tSize=%s \tDomain=%s\n"
                       % (len(self), self.domain.name) )
        if not self._constructed:
            ostream.write("\tNot constructed\n")
        elif None in self.keys():
            if None in self._data:
                ostream.write("\t%s\n" % ( value(self[None]), ))
            else:
                ostream.write("\tUndefined\n")
        else:
            for key, val in sorted(self.sparse_iteritems()):
                ostream.write("\t%s : %s\n" % (key, value(val)))
        if self._default_val is not None:
            ostream.write("\tdefault: ")
            if type(self._default_val) is types.FunctionType:
                ostream.write("(function)\n")
            else:
                ostream.write("%s\n" % self._default_val)

    def __len__(self):
        if self._default_val is None:
            return len(self._data)
        return len(self._index)

    def __contains__(self, ndx):
        if self._default_val is None:
            return ndx in self._data
        return ndx in self._index

    def __iter__(self):
        if self._default_val is None:
            return self._data.__iter__()
        return self._index.__iter__()

    #
    # NB: These are "sparse equivalent" access / iteration methods that
    # only loop over the defined data.
    #

    def sparse_keys(self):
        return list(iterkeys(self._data))

    def sparse_values(self):
        return [ self[x] for x in self._data ]

    def sparse_items(self):
        return [ (x, self[x]) for x in self._data ]

    def sparse_iterkeys(self):
        return iterkeys(self._data)

    def sparse_itervalues(self):
        for key in self._data:
            yield self[key]

    def sparse_iteritems(self):
        for key in self._data:
            yield (key, self[key])

    # TODO: Not sure what "reset" really means in this context...
    def reset(self):
        pass

    #
    # A utility to extract all index-value pairs defining this
    # parameter, returned as a dictionary. useful in many
    # contexts, in which key iteration and repeated __getitem__
    # calls are too expensive to extract the contents of a parameter.
    # NOTE: We are presently not careful if an index has not been
    #       explicitly defined - issues around defaults are bound
    #       to crop up.
    #
    def extract_values(self):
        # avoid the use of value() for performance reasons - it is painfully slow in this context.
        if self._mutable:
            return dict((key,val.value) for key,val in iteritems(self))
        else:
            return dict((key,value(val)) for key,val in iteritems(self))

    def _default(self, idx):
        # FIXME: Ideally, we should test for using an unconstructed
        # Param; however, actually enforcing this breaks lots & lots of
        # tests...
        #        
        #if not self._constructed:
        #    if idx is None:
        #        idx_str = '%s' % (self.name,)
        #    else:
        #        idx_str = '%s[%s]' % (self.name, idx,)
        #    raise ValueError(
        #        "Error retrieving Param value (%s): The Param value has "
        #        "not been constructed" % ( idx_str,) )                
        #
        # FIXME: Ideally, a key miss should raise a KeyError; however,
        # for mostly historical reasons, Pyomo has raised a ValueError
        # (the logic being that all indices used to be defined at
        # construction, so a key miss was the result of a bad value).
        # Changing this to a KeyError generates numerous test failures, so
        # when someone has time to track down and fix those errors, we
        # SHOULD change the exception to KeyError.
        if self._default_val is None:
            if idx is None:
                idx_str = '%s' % (self.name,)
            else:
                idx_str = '%s[%s]' % (self.name, idx,)
            raise ValueError(
                "Error retrieving Param value (%s): The Param value is "
                "undefined and no default value is specified"
                % ( idx_str,) )

        _default_type = type(self._default_val)
        if _default_type in native_types:
            val = self._default_val
        elif _default_type is types.FunctionType:
            val = apply_indexed_rule(
                self, self._default_val, self.parent(), idx )
        elif hasattr(self, '__getitem__'):
            try:
                val = self._default_val[idx]
            except IndexError:
                val = self._default_val
        else:
            val = self._default_val

        if self._mutable:
            # While slightly less efficient, calling __setitem__ ensures
            # that the parameter validation routine gets called in a
            # consistent manner.
            self[idx] = val
            return self[idx]
            #if self.is_indexed():
            #    return self._data.setdefault(
            #        idx, _ParamData( self, self.domain, val ) )
            #else:
            #    self.value = val
            #    self._data[None] = None
            #    return self
        else:
            return val

    def set_default(self, val):
        if self._constructed and not val is None and not type(val) is types.FunctionType and \
                val not in self.domain:
            raise ValueError(
                "Default value (%s) is not valid for Param domain %s" %
                ( str(val), self.domain.name ) )
        self._default_val = val

    def __setitem__(self, ndx, val):
        if self._constructed and not self._mutable:
            raise TypeError(
"""Attempting to set the value of the immutable parameter %s after the
parameter has been constructed.  If you intend to change the value of
this parameter dynamically, please declare the parameter as mutable
[i.e., Param(mutable=True)]""" % (self.name,))

        # Params should contain *values*.  Normally, I would just call
        # value(), but that forces the value to be a /numeric value/,
        # which for historical reasons, we have not forced.  Notably, we
        # have allowed Params with domain==Any to hold strings, tuples,
        # etc.  The following lets us use NumericValues to initialize
        # Params, but is optimized to check for "known" native types to
        # bypass a potentially expensive isinstance()==False call. 
        #
        if val.__class__ not in native_types:
            if isinstance(val, NumericValue):
                val = val()

        #
        # Validate the index
        #

        if self.nochecking:
            # Ironically, if we are *not* checking index values, then we
            # *must* flatten the incoming index tuple
            ndx = self.normalize_index(ndx)

        #
        # TBD: Potential optimization: if we find that updating a Param is
        # more common than setting it in the first place, then first
        # checking the _data and then falling back on the _index *might*
        # be more efficient.
        #
        elif ndx not in self._index:

            # We rely (for performance purposes) on "most" people doing things 
            # correctly; that is, they send us either a scalar or a valid tuple.  
            # So, for efficiency, we will check the index *first*, and only go
            # through the hassle of flattening things if the ndx is not found.
            ndx = self.normalize_index(ndx)
            if ndx not in self._index:
                if not self.is_indexed():
                    msg = "Error setting parameter value: " \
                          "Cannot treat the scalar Param '%s' as an array" \
                          % ( self.name, )
                else:
                    msg = "Error setting parameter value: " \
                          "Index '%s' is not valid for array Param '%s'" \
                          % ( ndx, self.name, )
                raise KeyError(msg)

        if ndx is None:
            self.value = val
            self._data[ndx] = None
        elif self._mutable:
            if ndx in self._data:
                self._data[ndx].value = val
            else:
                self._data[ndx] = _ParamData(self, self.domain, val)
        else:
            self._data[ndx] = val
        #
        # This is at the end, so the user can write their validation
        # tests as if the data was already added.
        #
        if self.nochecking:
            return
        if val not in self.domain:
            raise ValueError(
                "Invalid parameter value: %s[%s] = '%s', value type=%s.\n"
                "\tValue not in parameter domain %s" %
                ( self.name, ndx, val, type(val), self.domain.name ) )
        if self._validate is not None:
            if self.is_indexed():
                if type(ndx) is tuple:
                    tmp = ndx
                else:
                    tmp = (ndx,)
            else:
                tmp = ()
            if not apply_parameterized_indexed_rule(
                self, self._validate, self.parent(), val, tmp ):
                raise ValueError(
                    "Invalid parameter value: %s[%s] = '%s', value type=%s.\n"
                    "\tValue failed parameter validation rule" %
                    ( self.name, ndx, val, type(val) ) )

    def construct(self, data=None):
        """ Apply the rule to construct values in this set """

        if __debug__:
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug("Constructing Param, name=%s, from data=%s"
                             % ( self.name, str(data) ))

        if self._constructed:
            raise IOError(
                "Cannot reconstruct parameter '%s' (already constructed)"
                % self.name )
            return

        self.clear()

        #
        # Construct using the initial data or the data loaded from an
        # external source. Data values will be queried in to following
        # order:
        #   1) the 'data' dictionary
        #   2) the self._rule dictionary or rule
        #   3) [implicit: fall back on the default value]
        #
        # To accomplish this, we will first set all the values based on
        # self._rule, and then allow the data dictionary to
        # overwrite anything.
        #
        # NB: Singleton Params can always be treated as "normal" indexed
        # params, indexed by a set {None}.
        #

        #
        # NB: Previously, we would raise an exception for constructing
        # scalar parameters with no defined data.  As that was a special
        # case (i.e. didn't apply to arrays) and was frustrating for
        # Concrete folks who were going to initialize the value later,
        # we will allow an undefined Param to be constructed and will
        # instead throw an exception later if the user tries to *use* the
        # Param before it is initialized.
        #
        _init = self._rule
        if _init is not None:
            _init_type = type(_init)
            if _init_type is dict:
                for key, val in iteritems(_init):
                    self[key] = val
            elif _init_type is types.FunctionType:
                if self.is_indexed():
                    # a situation can arise in which the parameter is indexed by
                    # an empty set, or an empty set tuple. rare, but it has happened.
                    if len(self._index) != 0: 
                        _iter = self._index.__iter__()
                        idx = six.next(_iter)
                        val = apply_indexed_rule(self, _init, self.parent(), idx)
                        if type(val) is dict:
                            for key, v in iteritems(val):
                                self[key] = v
                        else:
                            # IMPORTANT: Do *not* call self.__setitem__ here
                            #    - we already know the index is valid (and
                            #    flattened), and validating indices is far too
                            #    expensive to waste time doing it
                            #    again. however, we still need to validate the
                            #    value generated by the rule, relative to the
                            #    parameter domain.
                            # REVISION: given the rewrite of __setitem__,
                            #    the index is not validated unless it is not
                            #    found in the corresponding data / index.  We
                            #    *could* reimplement the optimization metioned
                            #    above if it proves necessary, but until then,
                            #    the DRY principle dictates that we use
                            #    __setitem__.
                            self[idx] = val
                            for idx in _iter:
                                self[idx] = apply_indexed_rule(
                                    self, _init, self.parent(), idx )
                else:
                    self[None] = _init(self.parent())
            else:
                if isinstance(_init, NumericValue):
                    # Reduce NumericValues to scalars
                    _init = _init()
                elif isinstance(_init, SparseIndexedComponent):
                    # Ideally, we want to reduce SparseIndexedComponents
                    # to a dict, but without "densifying" it.  However,
                    # since there is no way to (easily) get the default
                    # value, we will take the "less surprising" route of
                    # letting the source become dense, so that we get
                    # the expected copy.
                    sparse_src = len(_init) != len(_init.keys())
                    tmp = {}
                    for key in _init.keys():
                        try:
                            val = _init[key]
                        except ValueError:
                            continue
                        tmp[key] = value(val)
                    tmp = dict(iteritems(_init))
                    if sparse_src and len(_init) == len(_init.keys()):
                        logger.warning("""
Initializing Param %s using a sparse mutable indexed component (%s).
    This has resulted in the conversion of the source to dense form.
""" % ( self.name, _init.name ) )
                    _init = tmp
                elif isinstance(_init, IndexedComponent):
                    # FIXME: is there a general form for
                    # IndexedComponent, or should we just wait until
                    # everything moves over to SparseIndexedComponents?
                    pass
                # if things looks like a dictionary, then we will treat
                # it as such
                _isDict = hasattr(_init, '__getitem__')
                if _isDict:
                    try:
                        for x in _init:
                            _init.__getitem__(x)
                    except:
                        _isDict = False
                if _isDict:
                    for key in _init:
                        self[key] = _init[key]
                else:
                    for key in self._index:
                        self[key] = _init
                
        if data is not None:
            try:
                for key, val in iteritems(data):
                    self[key] = val
            except Exception:
                msg = sys.exc_info()[1]
                if type(data) not in [dict]:
                   raise ValueError("Attempting to initialize parameter="+self.name+" with data="+str(data)+". Data type is not a dictionary, and a dictionary is expected. Did you create a dictionary with a \"None\" index?")
                else:
                    raise RuntimeError("Failed to set value for param="+str(self.name)+", index="+str(key)+", value="+str(val)+"; source error message="+str(msg))

        self._constructed = True
        self.set_default(self._default_val)


class _ParamElement(_ParamData, Param):

    def __init__(self, *args, **kwds):
        Param.__init__(self, *args, **kwds)
        _ParamData.__init__(self, self, self.domain, kwds.get('default',None))

    # Since this class derives from Component and Component.__getstate__
    # just packs up the entire __dict__ into the state dict, there s
    # nothng special that we need to do here.  We will just defer to the
    # super() get/set state.  Since all of our get/set state methods
    # rely on super() to traverse the MRO, this will automatically pick
    # up both the Component and Data base classes.
    #
    #def __getstate__(self):
    #    pass
    #
    #def __setstate__(self, state):
    #    pass

    def pprint(self, ostream=None, verbose=False):
        # Needed so that users find Param.pprint and not _ParamData.pprint
        Param.pprint(self, ostream=ostream, verbose=verbose)

    def __call__(self, *args, **kwds):
        # Needed because we rely on self[None] to know if this parameter
        # is valid.  In particular, if we are getting data from the
        # default value, calling self[None] will actually inject the
        # [None] entry into _data.
        ans = self[None]
        if None in self._data:
            return _ParamData.__call__(ans, *args, **kwds)
        else:
            return ans

    def set_value(self, value):
        if self._constructed and not self._mutable:
            raise TypeError(
"""Attempting to set the value of the immutable parameter %s after the
parameter has been constructed.  If you intend to change the value of
this parameter dynamically, please declare the parameter as mutable
[i.e., Param(mutable=True)]""" % (self.name,))
        self[None] = value
        
    def is_constant(self):
        return self._constructed and not self._mutable


class _ParamArray(Param):

    def __init__(self, *args, **kwds):
        Param.__init__(self, *args, **kwds)

    def display(self, ostream=None):
        self.pprint(ostream=ostream)


register_component(Param, "Parameter data that is used to define a model instance.")




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['BuildAction']

import logging
import types

from coopr.pyomo.base.component import register_component
from coopr.pyomo.base.indexed_component import IndexedComponent
from coopr.pyomo.base.misc import apply_indexed_rule

logger = logging.getLogger('coopr.pyomo')


class BuildAction(IndexedComponent):
    """A build action, which executes a rule for all valid indices.

    Constructor arguments:
        rule        The rule that is executed for every indice.

    Private class attributes:
        _rule       The rule that is executed for every indice.
    """

    def __init__(self, *args, **kwd):
        self._rule = kwd.pop('rule', None)
        kwd['ctype'] = BuildAction
        IndexedComponent.__init__(self, *args, **kwd)
        #
        if not type(self._rule) is types.FunctionType:
            raise ValueError("BuildAction must have an 'rule' option specified whose value is a function")

    def construct(self, data=None):
        """ Apply the rule to construct values in this set """
        if __debug__:
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug("Constructing Action, name="+self.name)
        #
        if self._constructed:
            return
        self._constructed=True
        #
        if None in self._index:
            # Singleton component
            self._rule(self._parent())
        else:
            # Indexed component
            for index in self._index:
                apply_indexed_rule(self, self._rule, self._parent(), index)


register_component(BuildAction, "This component is used to inject arbitrary actions into the model construction process.  The action rule is applied to every index value.")



import os
from os.path import join, dirname, abspath
import warnings

from coopr.pyomo import *
from coopr.opt import *
import pyutilib.th as unittest
from coopr.pyomo.tests.io import model_types
from coopr.pyomo.tests.io.writer_test_cases import testCases

thisDir = dirname(abspath( __file__ ))

# These are usually due to a bug in the latest version of the thirdparty solver
# Tests will be expected to fail. If they do not, that means the solver has been fixed
# and that particular case should no longer exist in the list of expected failures
ExpectedFailures = []
ExpectedFailures.append( ('cbc',
                          'lp',
                          model_types.duals_maximize,
                          "For a maximization problem where a variable is pushed to its "\
                          "lower bound, Cbc reports the reduced cost as a positive number. In "\
                          "practice this should be reported as a negative number. A ticket has "\
                          "been filed at:\nhttps://projects.coin-or.org/Cbc/ticket/125") )
ExpectedFailures.append( ('pico',
                          'nl',
                          model_types.duals_maximize,
                          "Pico classifies certain models with equality constraints as infeasible when "\
                          "using the NL file interface. A ticket has been filed.") )
ExpectedFailures.append( ('pico',
                          'nl',
                          model_types.duals_minimize,
                          "Pico classifies certain models with equality constraints as infeasible when "\
                          "using the NL file interface. A ticket has been filed.") )
ExpectedFailures.append( ('pico',
                          'nl',
                          model_types.inactive_index_LP,
                          "Pico reports the wrong objective function value.") )
ExpectedFailures.append( ('scip',
                          'nl',
                          model_types.simple_SOS2,
                          "SCIP (scipampl) does not recognize sos2 constraints inside NL files. "\
                          "A ticket has been filed.") )

def check_expected_failures(test_case, model_class):
    # If this situation is an expected failure then return the message why
    for case in ExpectedFailures:
        if (case[0] == test_case.name) and (case[1] == test_case.io) and (case[2] is model_class):
            return True, case[3]
    return False, ""

def CreateTestMethod(test_case, modelClass,symbolic_labels=False):
    
    # We do not want to test the plugin case on a model
    # class it is not capable of handling
    if modelClass().validateCapabilities(test_case) is False:
        return None

    def writer_test(self):
        # Instantiate the model class
        model_class = modelClass()
        save_filename = join(thisDir,test_case.name+"_"+test_case.io+"_"+model_class.descrStr()+".test.results")
        # cleanup possibly existing old test files
        try:
            os.remove(save_filename)
        except OSError:
            pass
        
        # Skip this test if the solver is not available on the system
        if test_case.available is False:
            #print test_case
            self.skipTest('Solver unavailable: '+test_case.name+' ('+test_case.io+')')
            return

        test_case.initialize()
        opt = test_case.solver

        if test_case.io == 'nl':
            self.assertEqual(opt.problem_format(), ProblemFormat.nl)
        elif test_case.io == 'lp':
            self.assertEqual(opt.problem_format(), ProblemFormat.cpxlp)
        elif test_case.io == 'python':
            self.assertEqual(opt.problem_format(), None)

        # check that the solver plugin is at least as capable as the
        # test_case advertises, otherwise the plugin capabilities need to be change
        # or the test case should be removed
        if not all(opt.has_capability(tag) is True for tag in test_case.capabilities):
            self.fail("Actual plugin capabilities are less than that of the "\
                      "of test case for the plugin: "+test_case.name+' ('+test_case.io+')')

        # Create the model instance and send to the solver
        model_class.generateModel()
        model = model_class.model
        self.assertTrue(model is not None)
        # Generates canonical repn for those writers that need it
        if test_case.io != 'nl':
            model.preprocess()

        # make sure we don't try to extract rc or dual suffixes from integer programs
        for suffix in test_case.import_suffixes:
            setattr(model,suffix,Suffix(direction=Suffix.IMPORT))

        # solve
        results = opt.solve(model,symbolic_solver_labels=symbolic_labels)
        model.load(results)
        model_class.saveCurrentSolution(save_filename,
                                        suffixes=test_case.import_suffixes)

        # There are certain cases where the latest solver version has a bug
        # so this should not cause a coopr test to fail
        is_expected_failure, failure_msg = check_expected_failures(test_case,modelClass)

        # validate the solution returned by the solver 
        rc = model_class.validateCurrentSolution(suffixes=test_case.import_suffixes)
        
        if is_expected_failure:
            if rc[0] is True:
                self.fail("\nThis test was marked as an expected failure but no failure occured. "\
                          "The reason given for the expected failure is:\n\n****\n"+failure_msg+"\n****\n\nPlease remove "\
                          "this case as an expected failure if the above issue has been corrected in "\
                          "the latest version of the solver.")
            else:
                warnings.warn( "\n\n**This test is failing but it has been marked as an expected failure "\
                               "due to a bug in the solver.\nSolver: %s\nWriter: %s\nModel:  %s\n"\
                                % (test_case.name,test_case.io,model_class.descrStr()) )
        elif rc[0] is False:
            self.fail("Solution mismatch for plugin "+test_case.name+' ('+test_case.io+") "\
                      "and problem type "+model_class.descrStr()+"\n"+rc[1]+"\n"+str(results.Solution(0)))

        # cleanup if the test passed
        try:
            os.remove(save_filename)
        except OSError:
            pass

    return writer_test

def addfntests(cls, tests, modelClass, symbolic_labels=False):
    for case in tests:
        test_method = CreateTestMethod(case, modelClass, symbolic_labels=symbolic_labels)
        if test_method is not None:
            if symbolic_labels is True:
                setattr(cls, "test_"+case.name+"_"+case.io+"_symbolic_labels", test_method)
            elif symbolic_labels is False:
                setattr(cls, "test_"+case.name+"_"+case.io+"_non_symbolic_labels", test_method)
            else:
                raise AssertionError("Bad value for create test method.")

class WriterTests_simple_LP(unittest.TestCase): pass
WriterTests_simple_LP = unittest.category('smoke','nightly','expensive')(WriterTests_simple_LP)
addfntests(WriterTests_simple_LP,testCases, model_types.simple_LP, symbolic_labels=False)
addfntests(WriterTests_simple_LP,testCases, model_types.simple_LP, symbolic_labels=True)

class WriterTests_block_LP(unittest.TestCase): pass
WriterTests_block_LP = unittest.category('smoke','nightly','expensive')(WriterTests_block_LP)
addfntests(WriterTests_block_LP,testCases, model_types.block_LP, symbolic_labels=False)
addfntests(WriterTests_block_LP,testCases, model_types.block_LP, symbolic_labels=True)

class WriterTests_inactive_index_LP(unittest.TestCase): pass
WriterTests_inactive_index_LP = unittest.category('smoke','nightly','expensive')(WriterTests_inactive_index_LP)
addfntests(WriterTests_inactive_index_LP,testCases, model_types.inactive_index_LP, symbolic_labels=False)
addfntests(WriterTests_inactive_index_LP,testCases, model_types.inactive_index_LP, symbolic_labels=True)

class WriterTests_simple_MILP(unittest.TestCase): pass
WriterTests_simple_MILP = unittest.category('nightly','expensive')(WriterTests_simple_MILP)
addfntests(WriterTests_simple_MILP,testCases, model_types.simple_MILP, symbolic_labels=False)
addfntests(WriterTests_simple_MILP,testCases, model_types.simple_MILP, symbolic_labels=True)

class WriterTests_simple_QP(unittest.TestCase): pass
WriterTests_simple_QP = unittest.category('nightly','expensive')(WriterTests_simple_QP)
addfntests(WriterTests_simple_QP,testCases, model_types.simple_QP, symbolic_labels=False)
addfntests(WriterTests_simple_QP,testCases, model_types.simple_QP, symbolic_labels=True)

class WriterTests_simple_MIQP(unittest.TestCase): pass
WriterTests_simple_MIQP = unittest.category('nightly','expensive')(WriterTests_simple_MIQP)
addfntests(WriterTests_simple_MIQP,testCases, model_types.simple_MIQP, symbolic_labels=False)
addfntests(WriterTests_simple_MIQP,testCases, model_types.simple_MIQP, symbolic_labels=True)

class WriterTests_simple_QCP(unittest.TestCase): pass
WriterTests_simple_QCP = unittest.category('nightly','expensive')(WriterTests_simple_QCP)
addfntests(WriterTests_simple_QCP,testCases, model_types.simple_QCP, symbolic_labels=False)
addfntests(WriterTests_simple_QCP,testCases, model_types.simple_QCP, symbolic_labels=True)

class WriterTests_simple_MIQCP(unittest.TestCase): pass
WriterTests_simple_MIQCP = unittest.category('nightly','expensive')(WriterTests_simple_MIQCP)
addfntests(WriterTests_simple_MIQCP,testCases, model_types.simple_MIQCP, symbolic_labels=False)
addfntests(WriterTests_simple_MIQCP,testCases, model_types.simple_MIQCP, symbolic_labels=True)

class WriterTests_simple_SOS1(unittest.TestCase): pass
WriterTests_simple_SOS1 = unittest.category('nightly','expensive')(WriterTests_simple_SOS1)
addfntests(WriterTests_simple_SOS1,testCases, model_types.simple_SOS1, symbolic_labels=False)
addfntests(WriterTests_simple_SOS1,testCases, model_types.simple_SOS1, symbolic_labels=True)

class WriterTests_simple_SOS2(unittest.TestCase): pass
WriterTests_simple_SOS2 = unittest.category('nightly','expensive')(WriterTests_simple_SOS2)
addfntests(WriterTests_simple_SOS2,testCases, model_types.simple_SOS2, symbolic_labels=False)
addfntests(WriterTests_simple_SOS2,testCases, model_types.simple_SOS2, symbolic_labels=True)

class WriterTests_duals_maximize(unittest.TestCase): pass
WriterTests_duals_maximize = unittest.category('nightly','expensive')(WriterTests_duals_maximize)
addfntests(WriterTests_duals_maximize,testCases, model_types.duals_maximize, symbolic_labels=False)
addfntests(WriterTests_duals_maximize,testCases, model_types.duals_maximize, symbolic_labels=True)

class WriterTests_duals_minimize(unittest.TestCase): pass
WriterTests_duals_minimize = unittest.category('nightly','expensive')(WriterTests_duals_minimize)
addfntests(WriterTests_duals_minimize,testCases, model_types.duals_minimize, symbolic_labels=False)
addfntests(WriterTests_duals_minimize,testCases, model_types.duals_minimize, symbolic_labels=True)

# A "solver should fail" test, which I am archiving for now
"""
            # If the solver is not capable of handling this
            # model class then we better get a failure here
            try:
                model.load(opt.solve(model))
                model_class.saveCurrentSolution(save_filename,
                                        suffixes=test_case.import_suffixes)
            except:
                pass
            else:
                # Okay so we may get to this point if we are using a 
                # plugin like ASL which must advertise having all capabilities
                # since it supports many solvers. And its possible that
                # sending something like a discrete model to ipopt can slip
                # through the cracks without error or warning. Hopefully the test 
                # case was set up so that the solution check will turn up bad.
                if model_class.validateCurrentSolution() is True:
                    warnings.warn("Plugin "+test_case.name+' ('+test_case.io+") is not capable of handling model class "+test_model_name+" "\
                                  "but no exception was thrown and solution matched baseline.")
"""

if __name__ == "__main__":
    unittest.main()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['Constraint', '_ConstraintData', 'ConstraintList', 'simple_constraint_rule', 'simple_constraintlist_rule']

import sys
import logging
import weakref

from inspect import isgenerator

from six import StringIO

import pyutilib.math
import pyutilib.misc

from coopr.pyomo.base.expr import Expression, generate_relational_expression, \
     _EqualityExpression, generate_expression_bypassCloneCheck
from coopr.pyomo.base.numvalue import ZeroConstant, value, as_numeric, _sub
from coopr.pyomo.base.component import ComponentData, register_component
from coopr.pyomo.base.indexed_component import IndexedComponent
from coopr.pyomo.base.misc import create_name, apply_indexed_rule
from coopr.pyomo.base.sets import Set
from coopr.pyomo.base.objective import Objective
from coopr.pyomo.base.component import Component

logger = logging.getLogger('coopr.pyomo')

_simple_constraint_rule_types = set([ type(None), bool ])


def simple_constraint_rule( fn ):
    """This is a decorator that translates None/True/False return values
    into Constraint.Skip/Constraint.Feasible/Constraint.Infeasible.
    This supports a simpler syntax in constraint rules, though these can be
    more difficult to debug when errors occur.

    Example use:

    @simple_constraint_rule
    def C_rule(model, i, j):
        ...
    """
    def wrapper_function ( *args, **kwargs ):
        if fn.__class__ in _simple_constraint_rule_types:
            value = fn
        else:
            value = fn( *args, **kwargs )
        if value.__class__ in _simple_constraint_rule_types:
            if value is None:
                return Constraint.Skip
            elif value is True:
                return Constraint.Feasible
            elif value is False:
                return Constraint.Infeasible
        return value
    return wrapper_function


def simple_constraintlist_rule( fn ):
    """This is a decorator that translates None/True/False return values
    into ConstraintList.End/Constraint.Feasible/Constraint.Infeasible.
    This supports a simpler syntax in constraint rules, though these can be
    more difficult to debug when errors occur.

    Example use:

    @simple_constraintlist_rule
    def C_rule(model, i, j):
        ...
    """
    def wrapper_function ( *args, **kwargs ):
        if fn.__class__ in _simple_constraint_rule_types:
            value = fn
        else:
            value = fn( *args, **kwargs )
        if value.__class__ in _simple_constraint_rule_types:
            if value is None:
                return ConstraintList.End
            elif value is True:
                return Constraint.Feasible
            elif value is False:
                return Constraint.Infeasible
        return value
    return wrapper_function



class _ConstraintData(ComponentData):
    """
    This class defines the data for a single constraint.

    Constructor arguments:
        name            The name of this constraint.  Use None for a default value.
        component       The Constraint object that owns this data.

    Public class attributes:
        active          A boolean that is true if this constraint is active in the model.
        component       The constraint component.
        body            The Pyomo expression for this constraint
        index           The index value for this constraint
        name            The name of this constraint

    """

    __pickle_slots__ = ( 'active', 'body', 'lower', 'upper', '_equality')
    __slots__ = __pickle_slots__ + ( '__weakref__', )

    def __init__(self, owner):

        ComponentData.__init__(self, owner)

        self.active = True

        self.body = None # an expression tree encoding

        self.lower = None
        self.upper = None
        self._equality = False

    def __getstate__(self):
        """
        This method must be defined because this class uses slots.
        """
        result = super(_ConstraintData, self).__getstate__()
        for i in _ConstraintData.__pickle_slots__:
            result[i] = getattr(self, i)
        return result

    # Since this class requires no special processing of the state
    # dictionary, it does not need to implement __setstate__()
    #def __setstate__(self, state):
    #    pass

    def __call__(self, exception=True):
        """
        Compute the value of the body of this constraint.

        This method does not simply return self.value because that data value may be out of date
        w.r.t. the value of decision variables.
        """

        if self.body is None:
            return None
        return self.body()

    def activate(self):
        self.active=True

    def deactivate(self):
        self.active=False

    def lslack(self):
        """
        Returns the value of L-f(x) for constraints of the form:
            L <= f(x) (<= U)
            (U >=) f(x) >= L
        """
        if self.lower is None:
            return float('-inf')
        else:
            return value(self.lower)-value(self.body)

    def uslack(self):
        """
        Returns the value of U-f(x) for constraints of the form:
            (L <=) f(x) <= U
            U >= f(x) (>= L)
        """
        if self.upper is None:
            return float('inf')
        else:
            return value(self.upper)-value(self.body)


class Constraint(IndexedComponent):
    """
    An object that defines a objective expression

    Construct an objective expression with rule to construct the
    expression

    keyword arguments:
    name: name of this object
    rule: function or rule definition of constraint
    expr: same as rule
    doc:  documentation string for constraint
    """

    NoConstraint    = (1000,)
    Skip            = (1000,)
    Infeasible      = (1001,)
    Violated        = (1001,)
    Feasible        = (1002,)
    Satisfied       = (1002,)

    def __new__(cls, *args, **kwds):
        if cls != Constraint:
            return super(Constraint, cls).__new__(cls)
        if args == ():
            return _ConstraintElement.__new__(_ConstraintElement)
        else:
            return _ConstraintArray.__new__(_ConstraintArray)

    def __init__(self, *args, **kwargs):
        tmprule = kwargs.pop('rule', None )
        tmprule = kwargs.pop('expr', tmprule )
        self.rule = tmprule
        self.trivial = False
        self._no_rule_init = kwargs.pop('noruleinit', None )
        self.domain = None
        self.value = None
        #
        kwargs.setdefault('ctype', Constraint)
        IndexedComponent.__init__(self, *args, **kwargs)

    def __str__(self):
        """
        Return a string representation of the constraint.  If the name attribute is None,
        then return ''.
        """
        if self.name is None:
            return ""
        else:
            return self.name

    def construct(self, data=None):
        """TODO"""
        #
        # cache the debug generation flag, to avoid the expense of
        # calling isEnabledFor() for each index - this is far too
        # expensive an operation to perform deep in a loop. the only
        # potential down-side is that the debug logging is either
        # disabled or enabled globally for the duration of this
        # method invocation - which doesn't seem like much of
        # (if any) limitation in practice.
        #
        generate_debug_messages = (__debug__ is True) and (logger.isEnabledFor(logging.DEBUG) is True)

        if generate_debug_messages is True:
            logger.debug("Constructing constraint %s",self.cname(True))

        if (self._no_rule_init is not None) and (self.rule is not None):
            logger.warn("noruleinit keyword is being used in conjunction with "
                        "rule keyword for constraint '%s'; defaulting to "
                        "rule-based construction.", self.cname(True))
        if self.rule is None:
            if self._no_rule_init is None:
                logger.warn("No construction rule or expression specified for "
                            "constraint '%s'", self.cname(True))
            else:
                self._constructed=True
            return

        if self._constructed:
            return

        self._constructed=True
        
        #
        # Local variables for code optimization
        #
        _self_rule = self.rule
        _self_parent = self._parent()
        #
        # a somewhat gory way to see if you have a singleton - the len()=1 check is 
        # needed to avoid "None" being passed illegally into set membership validation rules.
        #
        if (len(self._index) == 1) and (None in self._index): 
            if len(self._index) != 1:
                raise IndexError("Internal error: constructing constraint "\
                    "with both None and Index set")
            if generate_debug_messages:
                logger.debug("  Constructing single constraint (index=None)")
            if _self_rule.__class__ is bool or isinstance(_self_rule,Expression):
                expr = _self_rule
            else:
                expr = _self_rule(_self_parent)
            self.add(None, expr)
        else:
            if _self_rule.__class__ is bool or isinstance(_self_rule,Expression):
                raise IndexError("Cannot define multiple indices in a " \
                    "constraint with a single expression")
            for index in self._index:
                if generate_debug_messages:
                    logger.debug("  Constructing constraint index "+str(index))
                try:
                    expr = apply_indexed_rule( 
                        self, _self_rule, _self_parent, index )
                except Exception:
                    err = sys.exc_info()[1]
                    logger.error(
                        "Rule failed when generating expression for "
                        "constraint %s:\n%s: %s" 
                        % ( create_name(self.name, index), type(err).__name__, err ) )
                    raise 
                self.add( index, expr )

    def add(self, index, expr):
        # index: the constraint index (should probably be renamed)
        # expr: the constraint expression

        expr_type = expr.__class__

        # Convert deprecated expression values
        #
        if expr_type in _simple_constraint_rule_types:
            if expr is None:
                raise ValueError("""
Invalid constraint expression.  The constraint expression resolved to
None instead of a Pyomo object.  Please modify your rule to return
Constraint.Skip instead of None.

Error thrown for constaint "%s"
""" % ( create_name(self.name, index), ) )

            # There are cases where a user thinks they are generating a
            # valid 2-sided inequality, but Python's internal systems
            # for handling chained inequalities is doing something very
            # different and resolving it to True/False.  In this case,
            # chainedInequality will be non-None, but the expression
            # will be a bool.  For example, model.a < 1 > 0.
            if generate_relational_expression.chainedInequality is not None:
                buf = StringIO.StringIO()
                generate_relational_expression.chainedInequality.pprint(buf)
                # We are about to raise an exception, so it's OK to reset chainedInequality
                generate_relational_expression.chainedInequality = None
                raise ValueError("""
Invalid chained (2-sided) inequality detected.  The expression is
resolving to %s instead of a Pyomo Expression object.  This can occur
when the middle term of a chained inequality is a constant or immutable
parameter, for example, "model.a <= 1 >= 0".  The proper form for
2-sided inequalities is "0 <= model.a <= 1".

Error thrown for constaint "%s"

Unresolved (dangling) inequality expression:
    %s
""" % ( expr, create_name(self.name, index), buf ) )
            else:
                raise ValueError("""
Invalid constraint expression.  The constraint expression resolved to a
trivial Boolean (%s) instead of a Pyomo object.  Please modify your rule
to return Constraint.%s instead of %s.

Error thrown for constaint "%s"
""" % ( expr, expr and "Feasible" or "Infeasible", expr, create_name(self.name, index) ) )

        #
        # Ignore an 'empty' constraint
        #
        if expr_type is tuple and len(expr) == 1:
            if expr == Constraint.Skip or expr == Constraint.Feasible:
                return
            if expr == Constraint.Infeasible:
                raise ValueError( "Constraint '%s' is always infeasible" % 
                                  create_name(self.name,index) )
        #
        #
        # Local variables to optimize runtime performance
        #
        if index is None:
            conData = self
        else:
            conData = _ConstraintData(self)
        #
        if expr_type is tuple: # or expr_type is list:
            #
            # Form equality expression
            #
            if len(expr) == 2:
                arg0 = expr[0]
                if arg0 is not None:
                    arg0 = as_numeric(arg0)
                arg1 = expr[1]
                if arg1 is not None:
                    arg1 = as_numeric(arg1)

                conData._equality = True
                if arg1 is None or arg1.is_fixed():
                    conData.lower = conData.upper = arg1
                    conData.body = arg0
                elif arg0 is None or arg0.is_fixed():
                    conData.lower = conData.upper = arg0
                    conData.body = arg1
                else:
                    conData.lower = conData.upper = ZeroConstant
                    conData.body = arg0 - arg1
            #
            # Form inequality expression
            #
            elif len(expr) == 3:
                arg0 = expr[0]
                if arg0 is not None:
                    arg0 = as_numeric(arg0)
                    if not arg0.is_fixed():
                        raise ValueError(
                            "Constraint '%s' found a 3-tuple (lower, " 
                            "expression, upper) but the lower value was "
                            "non-constant." % create_name(self.name,index) )

                arg1 = expr[1]
                if arg1 is not None:
                    arg1 = as_numeric(arg1)

                arg2 = expr[2]
                if arg2 is not None:
                    arg2 = as_numeric(arg2)
                    if not arg2.is_fixed():
                        raise ValueError(
                            "Constraint '%s' found a 3-tuple (lower, " 
                            "expression, upper) but the upper value was "
                            "non-constant" % create_name(self.name,index) )

                conData.lower = arg0
                conData.body  = arg1
                conData.upper = arg2
            else:
                raise ValueError(
                    "Constructor rule for constraint '%s' returned a tuple" 
                    ' of length %d.  Expecting a tuple of length 2 or 3:\n' 
                    'Equality:   (left, right)\n' 
                    'Inequality: (lower, expression, upper)'
                    % ( create_name(self.name,index), len(expr) ))

            relational_expr = False
        else:
            try:
                relational_expr = expr.is_relational()
                if not relational_expr:
                    raise ValueError(
                        "Constraint '%s' does not have a proper value.  " 
                        "Found '%s'\nExpecting a tuple or equation.  "
                        "Examples:\n" 
                        "    summation( model.costs ) == model.income\n" 
                        "    (0, model.price[ item ], 50)"
                        % ( create_name(self.name,index), str(expr) ))
            except AttributeError:
                msg = "Constraint '%s' does not have a proper value.  " \
                      "Found '%s'\nExpecting a tuple or equation.  " \
                      "Examples:\n" \
                      "    summation( model.costs ) == model.income\n" \
                      "    (0, model.price[ item ], 50)" \
                      % ( create_name(self.name,index), str(expr) )
                if type(expr) is bool:
                    msg +="""
Note: constant Boolean expressions are not valid constraint expressions.
Some apparently non-constant compound inequalities (e.g. "expr >= 0 <= 1")
can return boolean values; the proper form for compound inequalities is
always "lb <= expr <= ub"."""
                raise ValueError(msg)
        #
        # Special check for chainedInequality errors like "if var < 1:"
        # within rules.  Catching them here allows us to provide the
        # user with better (and more immediate) debugging information.
        # We don't want to check earlier because we want to provide a
        # specific debugging message if the construction rule returned
        # True/False; for example, if the user did ( var < 1 > 0 )
        # (which also results in a non-None chainedInequality value)
        #
        if generate_relational_expression.chainedInequality is not None:
            from expr import chainedInequalityErrorMessage
            raise TypeError(chainedInequalityErrorMessage())
        #
        # Process relational expressions (i.e. explicit '==', '<', and '<=')
        #
        if relational_expr:
            if expr_type is _EqualityExpression:
                # Equality expression: only 2 arguments!
                conData._equality = True
                if expr._args[1].is_fixed():
                    conData.lower = conData.upper = expr._args[1]
                    conData.body = expr._args[0]
                elif expr._args[0].is_fixed():
                    conData.lower = conData.upper = expr._args[0]
                    conData.body = expr._args[1]
                else:
                    conData.lower = conData.upper = ZeroConstant
                    conData.body = generate_expression_bypassCloneCheck(_sub, expr._args[0], expr._args[1])
            else:
                # Inequality expression: 2 or 3 arguments
                if len(expr._args) == 3:
                    if not expr._args[0].is_fixed():
                        msg = "Constraint '%s' found a double-sided "\
                              "inequality expression (lower <= expression "\
                              "<= upper) but the lower bound was non-constant"
                        raise ValueError(msg % (create_name(self.name,index),))
                    if not expr._args[2].is_fixed():
                        msg = "Constraint '%s' found a double-sided "\
                              "inequality expression (lower <= expression "\
                              "<= upper) but the upper bound was non-constant"
                        raise ValueError(msg % (create_name(self.name,index),))
                    conData.lower = expr._args[0]
                    conData.body  = expr._args[1]
                    conData.upper = expr._args[2]
                else:
                    if expr._args[1].is_fixed():
                        conData.lower = None
                        conData.body  = expr._args[0]
                        conData.upper = expr._args[1]
                    elif expr._args[0].is_fixed():
                        conData.lower = expr._args[0]
                        conData.body  = expr._args[1]
                        conData.upper = None
                    else:
                        conData.lower = None
                        conData.body  = generate_expression_bypassCloneCheck(_sub, expr._args[0], expr._args[1])
                        conData.upper = ZeroConstant
        #
        # Replace numeric bound values with a NumericConstant object,
        # and reset the values to 'None' if they are 'infinite'
        #
        if conData.lower is not None:
            val = conData.lower()
            if not pyutilib.math.is_finite(val):
                if val > 0:
                    msg = "Constraint '%s' created with a +Inf lower bound"
                    raise ValueError(msg % ( create_name(self.name,index), ))
                conData.lower = None
            elif bool(val > 0) == bool(val <= 0):
                msg = "Constraint '%s' created with a non-numeric lower bound"
                raise ValueError(msg % ( create_name(self.name,index), ))
        if conData.upper is not None:
            val = conData.upper()
            if not pyutilib.math.is_finite(val):
                if val < 0:
                    msg = "Constraint '%s' created with a -Inf upper bound"
                    raise ValueError(msg % ( create_name(self.name,index), ))
                conData.upper = None
            elif bool(val > 0) == bool(val <= 0):
                msg = "Constraint '%s' created with a non-numeric upper bound"
                raise ValueError(msg % ( create_name(self.name,index), ))
        #
        # Error check, to ensure that we don't have a constraint that
        # doesn't depend on any variables / parameters
        #
        # Error check, to ensure that we don't have an equality constraint with
        # 'infinite' RHS
        #
        if conData._equality:
            if conData.lower != conData.upper: #pragma:nocover
                msg = "Equality constraint '%s' has non-equal lower and "\
                      "upper bounds (this is indicitive of a SERIOUS "\
                      "internal error in Pyomo)."
                raise RuntimeError(msg % create_name(self.name,index))
            if conData.lower is None:
                msg = "Equality constraint '%s' defined with non-finite term"
                raise ValueError(msg % create_name(self.name,index))
        #
        # hook up the constraint data object to the parent constraint.
        #
        self._data[index] = conData

    def pprint(self, ostream=None, verbose=False):
        """TODO"""
        if ostream is None:
            ostream = sys.stdout
        ostream.write("   "+self.name+" : ")
        if not self.doc is None:
            ostream.write(self.doc+'\n')
            ostream.write("  ")
        ostream.write("\tSize="+str(len(self._data.keys()))+' ')
        if isinstance(self._index,Set):
            ostream.write("\tIndex= "+self._index.name+'\n')
        else:
            ostream.write("\n")
        for val in self._data:
            if not val is None:
                ostream.write("\t"+str(val)+'\n')
            if self._data[val].lower is not None:
                ostream.write("\t\t")
                if self._data[val].lower.is_expression():
                    self._data[val].lower.pprint(ostream)
                else:
                    ostream.write(str(self._data[val].lower)+'\n')
            else:
                ostream.write("\t\t-Inf\n")
            ostream.write("\t\t<=\n")
            if self._data[val].body is not None:
                ostream.write("\t\t")
                if self._data[val].body.is_expression():
                    self._data[val].body.pprint(ostream)
                else:
                    ostream.write(str(self._data[val].body)+'\n')
            ostream.write("\t\t<=\n")
            if self._data[val].upper is not None:
                ostream.write("\t\t")
                if self._data[val].upper.is_expression():
                    self._data[val].upper.pprint(ostream)
                else:
                    ostream.write(str(self._data[val].upper)+'\n')
            elif self._data[val]._equality:
                ostream.write("\t\t")
                if self._data[val].lower.is_expression():
                    self._data[val].lower.pprint(ostream)
                else:
                    ostream.write(str(self._data[val].lower)+'\n')
            else:
                ostream.write("\t\tInf\n")

    def display(self, prefix="", ostream=None):
        """TODO"""
        if ostream is None:
            ostream = sys.stdout
        ostream.write(prefix+"Constraint "+self.name+" :")
        ostream.write("  Size="+str(len(self))+'\n')
        if None in self._data:
            if self._data[None].body is None:
                val = 'none'
            else:
                val = pyutilib.misc.format_io(self._data[None].body())
            ostream.write('%s  Value=%s\n' % (prefix, val))
        else:
            flag=True
            for key in self._data:
                if not self._data[key].active:
                    continue
                if flag:
                    ostream.write(prefix+"        \tLower\tBody\t\tUpper\n")
                    flag=False
                if self._data[key].lower is not None:
                    lval = str(self._data[key].lower())
                else:
                    lval = "-Infinity"
                val = str(self._data[key].body())
                if self._data[key].upper is not None:
                    uval = str(self._data[key].upper())
                else:
                    uval = "Infinity"
                ostream.write("%s  %s :\t%s\t%s\t%s\n" % (
                                 prefix, str(key), lval, val, uval ))
            if flag:
                ostream.write(prefix+"  None active\n")


class _ConstraintElement(Constraint, _ConstraintData):

    def __init__(self, *args, **kwd):
        _ConstraintData.__init__(self, self)
        Constraint.__init__(self, *args, **kwd)

    def __call__(self, exception=True):
        """Compute the value of the constraint body"""
        if self.body is None:
            return None
        return value(self.body)

    # all model components need to possess a "reset" method. because neither Constraint and
    # _ConstraintData are derived from NumericValue, the method is needed here. ultimately,
    # the right answer is to either nix the reset() from everywhere (it is not particularly
    # well-defined at the moment), or add it to the common Constraint base class.
    def reset(self):
        pass

    # Since this class derives from Component and Component.__getstate__
    # just packs up the entire __dict__ into the state dict, there s
    # nothng special that we need to do here.  We will just defer to the
    # super() get/set state.  Since all of our get/set state methods
    # rely on super() to traverse the MRO, this will automatically pick
    # up both the Component and Data base classes.
    #
    #def __getstate__(self):
    #    pass
    #
    #def __setstate__(self, state):
    #    pass


class _ConstraintArray(Constraint):

    def __call__(self, exception=True):
        """Compute the value of the constraint body"""
        if exception:
            msg = 'Cannot compute the value of an array of constraints'
            raise ValueError(msg)


class ConstraintList(_ConstraintArray):
    """
    A constraint component that represents a list of constraints.
    Constraints can be indexed by their index, but when they are added
    an index value is not specified.
    """

    End             = (1003,)

    def __init__(self, **kwargs):
        """Constructor"""
        #if args:
        #    raise ValueError("Cannot specify indices for a "
        #                     "ConstraintList object")
        #
        args = (Set(),)
        self._nconstraints = 0
        Constraint.__init__(self, *args, **kwargs)

    def construct(self, data=None):
        """TODO"""
        #
        if self._constructed:
            return
        self._index.construct()
        self._constructed=True

        if self.rule is None:
            return

        generate_debug_messages = (__debug__ is True) and \
                                  (logger.isEnabledFor(logging.DEBUG) is True)

        if generate_debug_messages is True:
            logger.debug("Constructing constraint list %s",self.name)

        if (self._no_rule_init is not None) and (self.rule is not None):
            msg = 'WARNING: noruleinit keyword is being used in conjunction ' \
                  "with rule keyword for constraint '%s'; defaulting to "     \
                  'rule-based construction.'
            print(msg % self.name)
        #
        # Local variables for code optimization
        #
        while True:
            val = self._nconstraints + 1
            if generate_debug_messages is True:
                logger.debug("   Constructing constraint index "+str(val))
            expr = apply_indexed_rule( self, self.rule, self._parent(), val )
            if not isgenerator(expr):
                expr = (expr,)
            for e in expr:
                if e is None:
                    #logger.warning(
                    #    "DEPRECATION WARNING: Constraint rule returned None "
                    #    "instead of ConstraintList.End" )
                    raise ValueError( "Constraint rule returned None "
                                      "instead of ConstraintList.End" )
                if (e.__class__ is tuple and e == ConstraintList.End):
                    #or (type(e) in (int, long, float) and e == 0):
                    return
                self.add(e)

    def add(self, expr):
        """TODO"""
        self._nconstraints += 1
        Constraint.add(self, self._nconstraints, expr)


register_component(Constraint, "Constraint expressions in a model.")
register_component(ConstraintList, "A list of constraints in a model.")




import os
from os.path import join, dirname, abspath

from coopr.pyomo import *

import json

thisDir = dirname(abspath( __file__ ))

class _ModelClassBase(object):
    """
    The base class for all model classifications
    """
    def __init__(self):
        self.linear = False
        self.integer = False
        self.quadratic_objective = False
        self.quadratic_constraint = False
        self.sos1 = False
        self.sos2 = False
        self.model = None
        self.results_file = None
        self.diff_tol = 1e-5

    def generateModel():
        raise NotImplementedError

    def saveCurrentSolution(self,filename,**kwds):
        assert self.model is not None
        model = self.model
        suffixes = dict((suffix,getattr(model,suffix)) for suffix in kwds.pop('suffixes',[]))
        for suf in suffixes.values():
            assert isinstance(suf,Suffix)
            assert suf.importEnabled() is True
        with open(filename,'w') as f:
            soln = {}
            for block in model.all_blocks():
                soln[block.cname(True)] = {}
                for suffix_name, suffix in suffixes.items():
                    if suffix.getValue(block) is not None:
                        soln[block.cname(True)][suffix_name] = suffix.getValue(block)
            for block in model.all_blocks():
                for var in active_subcomponents_data_generator(block,Var):
                    soln[var.cname(True)] = {}
                    soln[var.cname(True)]['value'] = value(var)
                    for suffix_name, suffix in suffixes.items():
                        if suffix.getValue(var) is not None:
                            soln[var.cname(True)][suffix_name] = suffix.getValue(var)
            for block in model.all_blocks():
                for con in active_subcomponents_data_generator(block,Constraint):
                    soln[con.cname(True)] = {}
                    soln[con.cname(True)]['value'] = con()
                    for suffix_name, suffix in suffixes.items():
                        if suffix.getValue(con) is not None:
                            soln[con.cname(True)][suffix_name] = suffix.getValue(con)
            for block in model.all_blocks():
                for obj in active_subcomponents_data_generator(block,Objective):
                    soln[obj.cname(True)] = {}
                    soln[obj.cname(True)]['value'] = value(obj)
                    for suffix_name, suffix in suffixes.items():
                        if suffix.getValue(obj) is not None:
                            soln[obj.cnaxme(True)][suffix_name] = suffix.getValue(obj)
            json.dump(soln, f, indent=2)

    def validateCurrentSolution(self,**kwds):
        assert self.model is not None
        assert self.results_file is not None
        model = self.model
        suffixes = dict((suffix,getattr(model,suffix)) for suffix in kwds.pop('suffixes',[]))
        for suf in suffixes.values():
            assert isinstance(suf,Suffix)
            assert suf.importEnabled() is True
        solution = None
        error_str = "Difference in solution for {0}.{1}:\n\tBaseline - {2}\n\tCurrent - {3}"
        with open(self.results_file,'r') as f:
            try:
                solution = json.load(f)
            except:
                return (False,"Problem reading file "+self.results_file)
        for block in model.all_blocks():
            for var in active_subcomponents_data_generator(block,Var):
                if not abs(solution[var.cname(True)]['value'] - value(var)) < self.diff_tol:
                    return (False, error_str.format(var.cname(True),'value',solution[var.cname(True)]['value'],value(var)))
                for suffix_name, suffix in suffixes.items():
                    if suffix_name in solution[var.cname(True)].keys():
                        if suffix.getValue(var) is None:
                            if not(solution[var.cname(True)][suffix_name] in solution["suffix defaults"][suffix_name]):
                                return (False, error_str.format(var.cname(True),suffix,solution[var.cname(True)][suffix_name],"none defined"))
                        elif not abs(solution[var.cname(True)][suffix_name] - suffix.getValue(var)) < self.diff_tol:
                            return (False, error_str.format(var.cname(True),suffix,solution[var.cname(True)][suffix_name],suffix.getValue(var)))
        for block in model.all_blocks():
            for con in active_subcomponents_data_generator(block,Constraint):
                if not abs(solution[con.cname(True)]['value'] - con()) < self.diff_tol:
                    return (False, error_str.format(con.cname(True),'value',solution[con.cname(True)]['value'],con()))
                for suffix_name, suffix in suffixes.items():
                    if suffix_name in solution[con.cname(True)].keys():
                        if suffix.getValue(con) is None:
                            if not (solution[con.cname(True)][suffix_name] in solution["suffix defaults"][suffix_name]):
                                return (False, error_str.format(con.cname(True),suffix,solution[con.cname(True)][suffix_name],"none defined"))
                        elif not abs(solution[con.cname(True)][suffix_name] - suffix.getValue(con)) < self.diff_tol:
                            return (False, error_str.format(con.cname(True),suffix,solution[con.cname(True)][suffix_name],suffix.getValue(con)))
        for block in model.all_blocks():
            for obj in active_subcomponents_data_generator(block,Objective):
                if not abs(solution[obj.cname(True)]['value'] - value(obj)) < self.diff_tol:
                    return (False, error_str.format(obj.cname(True),'value',solution[obj.cname(True)]['value'],value(obj)))
                for suffix_name, suffix in suffixes.items():
                    if suffix_name in solution[obj.cname(True)].keys(): 
                        if suffix.getValue(obj) is None:
                            if not(solution[obj.cname(True)][suffix_name] in solution["suffix defaults"][suffix_name]):
                                return (False, error_str.format(obj.cname(True),suffix,solution[obj.cname(True)][suffix_name],"none defined"))
                        elif not abs(solution[obj.cname(True)][suffix_name] - suffix.getValue(obj)) < self.diff_tol:
                            return (False, error_str.format(obj.cname(True),suffix,solution[obj.cname(True)][suffix_name],suffix.getValue(obj)))
        for block in model.all_blocks():
            for suffix_name, suffix in suffixes.items():
                if (solution[block.cname(True)] is not None) and (suffix in solution[block.cname(True)].keys()): 
                    if suffix.getValue(block) is None:
                        if not(solution[block.cname(True)][suffix_name] in solution["suffix defaults"][suffix_name]):
                            return (False, error_str.format(block.cname(True),suffix,solution[block.cname(True)][suffix_name],"none defined"))
                    elif not abs(solution[block.cname(True)][suffix_name] - suffix.getValue(block)) < sefl.diff_tol:
                        return (False, error_str.format(block.cname(True),suffix,solution[block.cname(True)][suffix_name],suffix.getValue(block)))
        return (True,"")
            
    def descrStr(self):
        raise NotImplementedError
    
    def validateCapabilities(self,opt):
        if (self.linear is True) and (not opt.has_capability('linear') is True):
            return False
        if (self.integer is True) and (not opt.has_capability('integer') is True):
            return False
        if (self.quadratic_objective is True) and (not opt.has_capability('quadratic_objective') is True):
            return False
        if (self.quadratic_constraint is True) and (not opt.has_capability('quadratic_constraint') is True):
            return False
        if (self.sos1 is True) and (not opt.has_capability('sos1') is True):
            return False
        if (self.sos2 is True) and (not opt.has_capability('sos2') is True):
            return False
        return True

class simple_LP(_ModelClassBase):
    """
    A continuous linear model
    """
    def __init__(self):
        _ModelClassBase.__init__(self)
        self.linear = True
        self.results_file = join(thisDir,"simple_LP.results")

    def descrStr(self):
        return "simple_LP"

    def generateModel(self):
        self.model = None
        self.model = ConcreteModel()
        model = self.model
        model.name = self.descrStr()
        
        model.a = Param(initialize=1.0)
        model.x = Var(within=NonNegativeReals)
        model.y = Var(within=NonNegativeReals)
        
        model.obj = Objective(expr=model.x + 3.0*model.y)
        model.c1 = Constraint(expr=model.a <= model.y)
        model.c2 = Constraint(expr=2.0 <= model.x/model.a - model.y <= 10)

class block_LP(_ModelClassBase):
    """
    A continuous linear model with nested blocks
    """
    def __init__(self):
        _ModelClassBase.__init__(self)
        self.linear = True
        self.results_file = join(thisDir,"block_LP.results")

    def descrStr(self):
        return "block_LP"

    def generateModel(self):
        self.model = None
        self.model = ConcreteModel()
        model = self.model
        model.name = self.descrStr()
        
        model.b = Block()
        model.B = Block([1,2,3])
        model.a = Param(initialize=1.0)
        model.b.x = Var(within=NonNegativeReals)
        model.B[1].x = Var(within=NonNegativeReals)
        
        model.obj = Objective(expr=model.b.x + 3.0*model.B[1].x)
        model.B[2].c = Constraint(expr=model.a <= model.B[1].x)
        model.B[3].c = Constraint(expr=2.0 <= model.b.x/model.a - model.B[1].x <= 10)

class inactive_index_LP(_ModelClassBase):
    """
    A continuous linear model where component subindices have been deactivated
    """
    def __init__(self):
        _ModelClassBase.__init__(self)
        self.linear = True
        self.results_file = join(thisDir,"inactive_index_LP.results")

    def descrStr(self):
        return "inactive_index_LP"

    def generateModel(self):
        self.model = None
        self.model = ConcreteModel()
        model = self.model
        model.name = self.descrStr()
        
        model.s = Set(initialize=[1,2])
        model.x = Var()
        model.y = Var()
        
        def obj_rule(model,i):
            if i == 1:
                return model.x-model.y
            else:
                return -model.x+model.y
        model.obj = Objective(model.s,rule=obj_rule)
        model.OBJ = Objective(expr=model.x+model.y)
        model.obj[1].deactivate()
        model.OBJ.deactivate()
        model.c1 = ConstraintList()
        model.c1.add(model.x<=1)   # index=1
        model.c1.add(model.x>=-1)  # index=2
        model.c1.add(model.y<=1)   # index=3
        model.c1.add(model.y>=-1)  # index=4
        model.c1[1].deactivate()
        model.c1[4].deactivate()
        def c2_rule(model,i):
            if i == 1:
                return model.y >= -2
            else:
                return model.x <= 2
        model.c2 = Constraint(model.s)

class simple_MILP(_ModelClassBase):
    """
    A mixed-integer linear model
    """
    def __init__(self):
        _ModelClassBase.__init__(self)
        self.linear = True
        self.integer = True
        self.results_file = join(thisDir,"simple_MILP.results")

    def descrStr(self):
        return "simple_MILP"

    def generateModel(self):
        self.model = None
        self.model = ConcreteModel()
        model = self.model
        model.name = self.descrStr()
    
        model.a = Param(initialize=1.0)
        model.x = Var(within=NonNegativeReals)
        model.y = Var(within=Binary)
        
        model.obj = Objective(expr=model.x + 3.0*model.y)
        model.c1 = Constraint(expr=model.a <= model.y)
        model.c2 = Constraint(expr=2.0 <= model.x/model.a - model.y <= 10)

class simple_QP(_ModelClassBase):
    """
    A continuous model with a quadratic objective and linear constraints
    """
    def __init__(self):
        _ModelClassBase.__init__(self)
        self.linear = True
        self.quadratic_objective = True
        self.results_file = join(thisDir,"simple_QP.results")

    def descrStr(self):
        return "simple_QP"

    def generateModel(self):
        self.model = None
        self.model = ConcreteModel()
        model = self.model
        model.name = self.descrStr()
        
        model.a = Param(initialize=1.0)
        model.x = Var(within=NonNegativeReals)
        model.y = Var(within=NonNegativeReals)
        
        model.obj = Objective(expr=model.x**2 + 3.0*model.y**2)
        model.c1 = Constraint(expr=model.a <= model.y)
        model.c2 = Constraint(expr=2.0 <= model.x/model.a - model.y <= 10)

class simple_MIQP(_ModelClassBase):
    """
    A mixed-integer model with a quadratic objective and linear constraints
    """
    def __init__(self):
        _ModelClassBase.__init__(self)
        self.linear = True
        self.integer = True
        self.quadratic_objective = True
        self.results_file = join(thisDir,"simple_MIQP.results")

    def descrStr(self):
        return "simple_MIQP"

    def generateModel(self):
        self.model = None
        self.model = ConcreteModel()
        model = self.model
        model.name = self.descrStr()
        
        model.a = Param(initialize=1.0)
        model.x = Var(within=NonNegativeReals)
        model.y = Var(within=Binary)
        
        model.obj = Objective(expr=model.x**2 + 3.0*model.y**2)
        model.c1 = Constraint(expr=model.a <= model.y)
        model.c2 = Constraint(expr=2.0 <= model.x/model.a - model.y <= 10)

class simple_QCP(_ModelClassBase):
    """
    A continuous model with a quadratic objective and quadratics constraints
    """
    def __init__(self):
        _ModelClassBase.__init__(self)
        self.linear = True
        self.quadratic_objective = True
        self.quadratic_constraint = True
        self.results_file = join(thisDir,"simple_QCP.results")

    def descrStr(self):
        return "simple_QCP"

    def generateModel(self):
        self.model = None
        self.model = ConcreteModel()
        model = self.model
        model.name = self.descrStr()

        model.x = Var(within=NonNegativeReals)
        model.y = Var(within=NonNegativeReals)
        model.z = Var(within=NonNegativeReals)

        model.obj = Objective(expr=model.x,sense=maximize)
        model.c0 = Constraint(expr=model.x+model.y+model.z == 1)
        model.qc0 = Constraint(expr=model.x**2 + model.y**2 <= model.z**2)
        model.qc1 = Constraint(expr=model.x**2 <= model.y*model.z)

class simple_MIQCP(_ModelClassBase):
    """
    A mixed-integer model with a quadratic objective and quadratic constraints
    """
    def __init__(self):
        _ModelClassBase.__init__(self)
        self.linear = True
        self.integer = True
        self.quadratic_objective = True
        self.quadratic_constraint = True
        self.results_file = join(thisDir,"simple_MIQCP.results")

    def descrStr(self):
        return "simple_MIQCP"

    def generateModel(self):
        self.model = None
        self.model = ConcreteModel()
        model = self.model
        model.name = self.descrStr()

        model.x = Var(within=Binary)
        model.y = Var(within=Binary)
        model.z = Var(within=Binary)

        model.obj = Objective(expr=model.x,sense=maximize)
        model.c0 = Constraint(expr=model.x+model.y+model.z == 1)
        model.qc0 = Constraint(expr=model.x**2 + model.y**2 <= model.z**2)
        model.qc1 = Constraint(expr=model.x**2 <= model.y*model.z)

class simple_SOS1(_ModelClassBase):
    """
    A discrete linear model with sos1 constraints
    """
    def __init__(self):
        _ModelClassBase.__init__(self)
        self.linear = True
        self.integer = True
        self.sos1 = True
        self.results_file = join(thisDir,"simple_SOS1.results")

    def descrStr(self):
        return "simple_SOS1"

    def generateModel(self):
        self.model = None
        self.model = ConcreteModel()
        model = self.model
        model.name = self.descrStr()

        model.a = Param(initialize=0.1)
        model.x = Var(within=NonNegativeReals)
        model.y = Var([1,2],within=NonNegativeReals)
        
        model.obj = Objective(expr=model.x + model.y[1]+2*model.y[2])
        model.c1 = Constraint(expr=model.a <= model.y[2])
        model.c2 = Constraint(expr=2.0 <= model.x <= 10.0)
        model.c3 = SOSConstraint(var=model.y, set=[1,2], sos=1)
        model.c4 = Constraint(expr=summation(model.y) == 1)

class simple_SOS2(_ModelClassBase):
    """
    A discrete linear model with sos2 constraints
    """
    def __init__(self):
        _ModelClassBase.__init__(self)
        self.linear = True
        self.integer = True
        self.sos2 = True
        self.results_file = join(thisDir,"simple_SOS2.results")

    def descrStr(self):
        return "simple_SOS2"

    def generateModel(self):
        self.model = None
        self.model = ConcreteModel()
        model = self.model
        model.name = self.descrStr()

        model.f = Var()
        model.x = Var(bounds=(1,3))
        model.fi = Param([1,2,3],mutable=True)
        model.fi[1] = 1.0
        model.fi[2] = 2.0
        model.fi[3] = 0.0
        model.xi = Param([1,2,3],mutable=True)
        model.xi[1] = 1.0
        model.xi[2] = 2.0
        model.xi[3] = 3.0
        model.p = Var(within=NonNegativeReals)
        model.n = Var(within=NonNegativeReals)
        model.lmbda = Var([1,2,3])
        model.obj = Objective(expr=model.p+model.n)
        model.c1 = ConstraintList()
        model.c1.add(0.0 <= model.lmbda[1] <= 1.0)
        model.c1.add(0.0 <= model.lmbda[2] <= 1.0)
        model.c1.add(0.0 <= model.lmbda[3])
        model.c2 = SOSConstraint(var=model.lmbda, set=[1,2,3], sos=2)
        model.c3 = Constraint(expr=summation(model.lmbda) == 1)
        model.c4 = Constraint(expr=model.f==summation(model.fi,model.lmbda))
        model.c5 = Constraint(expr=model.x==summation(model.xi,model.lmbda))
        model.x = 2.75
        model.x.fixed = True

class duals_maximize(_ModelClassBase):
    """
    A discrete linear model with sos2 constraints
    """
    def __init__(self):
        _ModelClassBase.__init__(self)
        self.linear = True
        self.results_file = join(thisDir,"duals_maximize.results")

    def descrStr(self):
        return "duals_maximize"

    def generateModel(self):
        self.model = None
        self.model = ConcreteModel()
        model = self.model
        model.name = self.descrStr()

        model.x = Var(RangeSet(1,12))
        model.x[1].setlb(-1)
        model.x[1].setub(1)
        model.x[2].setlb(-1)
        model.x[2].setub(1)
        model.obj = Objective(expr=sum(model.x[i]*((-1)**(i)) for i in model.x.index_set()),sense=maximize)
        model.c = ConstraintList()
        # to make the variable used in the constraint match the name
        model.c.add(Constraint.Skip)
        model.c.add(Constraint.Skip)
        model.c.add(model.x[3]>=-1.)
        model.c.add(model.x[4]<=1.)
        model.c.add(model.x[5]==-1.)
        model.c.add(model.x[6]==-1.)
        model.c.add(model.x[7]==1.)
        model.c.add(model.x[8]==1.)
        model.c.add((-1.,model.x[9],-1.))
        model.c.add((-1.,model.x[10],-1.))
        model.c.add((1.,model.x[11],1.))
        model.c.add((1.,model.x[12],1.))

        model.c_inactive = ConstraintList()
        # to make the variable used in the constraint match the name
        model.c_inactive.add(Constraint.Skip)
        model.c_inactive.add(Constraint.Skip)
        model.c_inactive.add(model.x[3]>=-2.)
        model.c_inactive.add(model.x[4]<=2.)

class duals_minimize(_ModelClassBase):
    """
    A discrete linear model with sos2 constraints
    """
    def __init__(self):
        _ModelClassBase.__init__(self)
        self.linear = True
        self.results_file = join(thisDir,"duals_minimize.results")

    def descrStr(self):
        return "duals_minimize"

    def generateModel(self):
        self.model = None
        self.model = ConcreteModel()
        model = self.model
        model.name = self.descrStr()

        model.x = Var(RangeSet(1,12))
        model.x[1].setlb(-1)
        model.x[1].setub(1)
        model.x[2].setlb(-1)
        model.x[2].setub(1)
        model.obj = Objective(expr=sum(model.x[i]*((-1)**(i+1)) for i in model.x.index_set()))
        model.c = ConstraintList()
        # to make the variable used in the constraint match the name
        model.c.add(Constraint.Skip)
        model.c.add(Constraint.Skip)
        model.c.add(model.x[3]>=-1.)
        model.c.add(model.x[4]<=1.)
        model.c.add(model.x[5]==-1.)
        model.c.add(model.x[6]==-1.)
        model.c.add(model.x[7]==1.)
        model.c.add(model.x[8]==1.)
        model.c.add((-1.,model.x[9],-1.))
        model.c.add((-1.,model.x[10],-1.))
        model.c.add((1.,model.x[11],1.))
        model.c.add((1.,model.x[12],1.))

        model.c_inactive = ConstraintList()
        # to make the variable used in the constraint match the name
        model.c_inactive.add(Constraint.Skip)
        model.c_inactive.add(Constraint.Skip)
        model.c_inactive.add(model.x[3]>=-2.)
        model.c_inactive.add(model.x[4]<=2.)

if __name__ == "__main__":
    from coopr.opt import *
    M = simple_QP()
    M.generateModel()
    model = M.model
    model.dual = Suffix(direction=Suffix.IMPORT)
    model.rc = Suffix(direction=Suffix.IMPORT)
    #model.slack = Suffix(direction=Suffix.IMPORT)

    model.preprocess()
    model.write(format=None,filename="junk.nl",symbolic_solver_labels=True)
    
    opt = SolverFactory("cplexamp")
    
    
    
    results = opt.solve(model)#,keepfiles=True,symbolic_solver_labels=True,tee=True)
    
    #print(results)
    #print(model.update_results(results))
    #model.load(results)
    #M.saveCurrentSolution("junk",suffixes=['dual','rc','slack'])
    #print(M.validateCurrentSolution(suffixes=['dual','rc','slack']))





#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['DataPortal']

import logging
from coopr.pyomo.base.plugin import *

logger = logging.getLogger('coopr.pyomo')

class DataPortal(object):
    """
    An object that manages data imports and exports from
    external data sources.

    This object contains the interface routines for importing and
    exporting data from external data sources.
    """

    def __init__(self, **kwds):
        """
        Constructor
        """
        # Initialize this object with no data manager
        self.data_manager = None

        # maps initialization data as follows: _data[namespace][symbol] -> data
        self._data={}

        # This is the data that is imported from various sources
        self._default={}

        # the model for which this data is associated. used for error
        # checking, e.g., object names must exist in the model, set 
        # dimensions must match, etc.
        self._model = kwds.pop('model', None)

        if 'filename' in kwds:
            filename = kwds.pop('filename')
            self.connect(filename=filename, **kwds)
            self.load()
            self.disconnect()
        elif 'data_dict' in kwds:
            data = kwds.pop('data_dict')
            if not data is None:
                self._data = data
        elif len(kwds) > 0:
            raise ValueError("Unknown options: %s" % str(kwds.keys()))

    #
    # intent of this method is to add, on a component-by-component 
    # basis, initialization data to a ModelData instance. not the
    # "usual" form of data initialization.
    #
    def Xinitialize(self, component, data, namespace=None):
        if namespace not in self._data.keys():
            self._data[namespace]={}
        self._data[namespace][component] = data

    def connect(self, **kwds):
        """
        Construct a data source objects that is associated with the input source.
        This data manager is used to process future data imports and exports.
        """
        if not self.data_manager is None:
            self.data_manager.close()
        data = kwds.get('filename',None)
        if data is None:
            data = kwds.get('server',None)
        tmp = data.split(".")[-1]
        self.data_manager = DataManagerFactory(tmp) 
        if self.data_manager is None:
            raise IOError("Unknown file format '%s'" % tmp)
        self.data_manager.initialize(**kwds)
        self.data_manager.open()

    def disconnect(self):
        """
        Close the Construct a data source objects that is associated with the input source.
        This data manager is used to process future data imports and exports.
        """
        self.data_manager.close()
        self.data_manager = None
        
    def load(self, **kwds):
        """
        Import data from an external data source.
        """
        _model = kwds.pop('model', None)
        if not _model is None:
            self._model=_model
        if __debug__:
            logger.debug("Loading data...")
        _disconnect=False
        if self.data_manager is None:
            self.connect(**kwds)
            _disconnect=True
        elif len(kwds) > 0:
            self.data_manager.add_options(**kwds)
        self._preprocess()
        if not self.data_manager.read():
            print("Warning: error occured while processing %s" % str(self.data_manager))
        if __debug__:
            logger.debug("Processing data ...")
        status = self.data_manager.process(self._model, self._data, self._default)
        self.data_manager.clear()
        if _disconnect:
            self.disconnect()
        if __debug__:
            logger.debug("Done.")

    def store(self, **kwds):
        """
        Export data from to an external data source.
        """
        if __debug__:
            logger.debug("Storing data...")
        _disconnect=False
        if self.data_manager is None:
            self.connect(**kwds)
            _disconnect=True
        elif len(kwds) > 0:
            self.data_manager.add_options(**kwds)
        self._preprocess()
        self.load_data()
        if not self.data_manager.write(self._data):
            print("Warning: error occured while processing %s" % str(self.data_manager))
        if _disconnect:
            self.disconnect()
        if __debug__:
            logger.debug("Done.")

    def data(self, name, namespace=None):
        if not namespace in self._data:
            raise IOError("Unknown namespace '%s'" % str(namespace))
        ans = self._data[namespace][name]
        if None in ans:
            return ans[None]
        return ans

    def _preprocess(self):
        options = self.data_manager.options
        #
        if options.data is None and (not options.set is None or not options.param is None or not options.index is None):
            options.data = []
            if not options.set is None:
                if type(options.set) in (list,tuple):
                    for item in options.set:
                        options.data.append(item)
                else:
                    options.data.append(options.set)
            if not options.index is None:
                options.data.append(options.index)
            if not options.param is None:
                if type(options.param) in (list,tuple):
                    for item in options.param:
                        options.data.append(item)
                else:
                    options.data.append(options.param)
        #
        if not options.data is None:
            if type(options.data) in (list, tuple):
                ans = []
                for item in options.data:
                    try:
                        ans.append(item.name)
                        self._model = item.model()
                    except:
                        ans.append(item)
                options.data = ans
            else:
                try:
                    self._model = options.data.model()
                    options.data = self.data_manager.options.data.name
                except:
                    pass


    def load_data(self):
        if not self.data_manager.options.data is None:
            if not self._model is None:
                if type(self.data_manager.options.data) is list:
                    for name in self.data_manager.options.data:
                        c = getattr(self._model, name)
                        try:
                            self._data[name] = c.data()
                        except:
                            self._data[name] = c.extract_values()
                else:
                    name = self.data_manager.options.data
                    c = getattr(self._model, name)
                    try:
                        self._data[name] = c.data()
                    except:
                        self._data[name] = c.extract_values()
 



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

import types
from coopr.pyomo.base.numvalue import as_numeric
from coopr.pyomo.base.component import Component
from coopr.pyomo.base.expr import Expression, clone_expression, _ExternalFunctionExpression

__all__  = ( 'ExternalFunction', )


class ExternalFunction(Component):
    def __init__(self, **kwds):
        self._library = kwds.pop('library', None)
        self._function = kwds.pop('function', None)
        kwds.setdefault('ctype', ExternalFunction)
        Component.__init__(self, **kwds)
        self._constructed = True
        ### HACK ###
        # FIXME: We must declare an _index attribute because
        # block._add_temporary_set assumes ALL components define an
        # index.  Sigh.
        self._index = None

    def __call__(self, *args):
        idxs = range(len(args))
        idxs.reverse()
        for i in idxs:
            if type(args[i]) is types.GeneratorType:
                args = args[:i] + tuple(args[i]) + args[i+1:]
        return _ExternalFunctionExpression( self, tuple(
                _external_fcn__clone_if_needed(x) if isinstance(x, Expression)
                else x if isinstance(x, basestring)
                else as_numeric(x)
                for x in args ) )

    def cname(self, fully_qualified=False, name_buffer=None):
        if self.name:
            return super(ExternalFunction, self).cname(
                fully_qualified, name_buffer )
        else:
            return str(self._library) + ":" + str(self._function)
        



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['IndexedComponent']

from six import iterkeys, itervalues, iteritems

from coopr.pyomo.base.component import Component
from coopr.pyomo.base.sets import Set


def process_setarg(arg):
    """
    Process argument and return an associated set object.
    """
    if isinstance(arg,Set):
        # Argument is a Set instance
        return arg
    elif isinstance(arg,Component):
        raise TypeError("Cannot index a component with a non-set component")
    else:
        try:
            # Argument has set_options attribute, which is used to initialize the set
            options = getattr(arg,'set_options')
            options['initialize'] = arg
            return Set(**options)
        except:
            pass
    # Argument is assumed to be an initialization function
    return Set(initialize=arg)


class IndexedComponent(Component):
    """
    This is the base class for all indexed modeling components.

    Constructor arguments:
        ctype       The class type for the derived subclass
        doc         A text string describing this component

    Private class attributes:
        _data       A dictionary from the index set to component data objects
        _index      The set of valid indices
        _index_set  A tuple of set objects that represents the index set
        _ndim       The dimension of the index set
    """

    def __init__(self, *args, **kwds):
        Component.__init__(self, **kwds)
        #
        self._data = {}
        #
        if args == ():
            #
            # If no indexing sets are provided, generate a dummy index
            #
            self._index = {None:None}
            self._index_set = None
            self._ndim = 0
        elif len(args) == 1:
            #
            # If a single argument is provided, the define the index using that
            # argument
            #
            self._index = process_setarg(args[0])
            self._index_set = None
            #
            # Compute the dimension of the indexing sets
            #
            self._compute_dim()
        else:
            #
            # If multiple arguments are provided, define _index_set with a tuple
            # of set objects
            #
            self._index = {}
            self._index_set=tuple( process_setarg(arg) for arg in args )
            #
            # Compute the dimension of the indexing sets
            #
            self._compute_dim()

    def clear(self):
        """Clear the data in this component"""
        self._data = {}

    def __len__(self):
        return len(self._data)

    def __contains__(self, ndx):
        return ndx in self._data

    def keys(self):
        return self._data.keys()

    def values(self):
        return self._data.values()

    def items(self):
        return self._data.items()

    def __iter__(self):
        return self._data.__iter__()

    def iteritems(self):
        return iteritems(self._data)

    def iterkeys(self):
        return iterkeys(self._data)

    def itervalues(self):
        return itervalues(self._data)

    def __getitem__(self, ndx):
        """
        This method returns the data corresponding to the given index.
        """
        if ndx in self._data:
            return self._data[ndx]
        raise KeyError("Unknown index in component '%s': %s" % ( self.name, str(ndx) ))

    def index_set(self):
        """Return the index set"""
        return self._index

    def is_indexed(self):
        """Return true if this component is indexed"""
        return self._ndim > 0

    def dim(self):
        """Return the dimension of the index"""
        return self._ndim

    def _compute_dim(self):
        """Compute the dimension of the set"""
        if self._index_set is None:
            # A single index set
            #if type(self._index) is dict:
            #    # If the index set is dictionary, then the argument list was empty
            #    self._ndim = 0
            #else:
            #    # If the index set is Pyomo Set, then use its dimen attribute
            self._ndim = self._index.dimen
        else:
            # A multi-set index
            self._ndim = 0
            for iset in self._index_set:
                # Each index set is a Pyomo Set, so use its dimen attribute
                self._ndim += iset.dimen




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['Var', 'VarList', '_VarData']

import logging
import types
import sys
import weakref
from six import iteritems, iterkeys, itervalues

import pyutilib.math
import pyutilib.misc
from pyutilib.enum import Enum

from coopr.pyomo.base.numvalue import NumericValue, value, as_numeric, is_fixed
from coopr.pyomo.base.set_types import BooleanSet, IntegerSet, Reals
from coopr.pyomo.base.component import Component, ComponentData, register_component
from coopr.pyomo.base.indexed_component import IndexedComponent
from coopr.pyomo.base.misc import apply_indexed_rule
from coopr.pyomo.base.misc import create_name
from coopr.pyomo.base.sets import Set, _SetContainer
from coopr.pyomo.base.util import isfunctor

try:
    xrange = xrange
except:
    xrange = range

logger = logging.getLogger('coopr.pyomo')

class _VarData(ComponentData, NumericValue):
    """
    This class defines the data for a single variable.

    Constructor Arguments:
        name        The name of this variable.  Use None for a default value.
        domain      The domain of this variable.  Use None for a default value.
        component   The Var object that owns this data.

    Public Class Attributes:
        active      If True, then this variable is active in the model.
        domain      The domain of this variable.
        fixed       If True, then this variable is treated as a fixed constant in the model.
        initial     The default initial value for this variable.
        lb          A lower bound for this variable.  The lower bound can be either 
                        numeric constants, parameter values, expressions or any object that 
                        can be called with no arguments.
        ub          A upper bound for this variable.  The upper bound can be either 
                        numeric constants, parameter values, expressions or any object that 
                        can be called with no arguments.
        name        A text string that provides an easy-to-read representation of the
                        variable/index pair (e.g. var[x,y,z])
        stale       A Boolean indicating whether the value of this variable is legitimiate -
                    or, in other words, if it should be considered legitimate for purposes 
                    of reporting or other interrogation. 
        value       The numeric value of this variable.
    """

    __pickle_slots__ = ('value','domain','initial','active','lb','ub','fixed','stale')
    __slots__ = __pickle_slots__ + ( '__weakref__', )

    def __init__(self, component, domain):
        """
        Constructor
        """
        ComponentData.__init__(self, component)
        # The following three lines are equivalent to calling the
        # basic NumericValue constructor, i.e., as follows:
        # NumericValue.__init__(self, name, domain, None, False)
        self.domain = domain
        self.value = None
        #
        self.active = True
        self.fixed = False
        self.initial = None
        # IMPT: The type of the lower and upper bound attributes can either be atomic numeric types
        #       in Python, expressions, etc. Basically, they can be anything that passes an "is_fixed" test.
        self.lb = None
        self.ub = None

        self.stale = True


    def __getstate__(self):
        """
        This method must be defined because this class uses slots.
        """
        result = super(_VarData, self).__getstate__()
        for i in _VarData.__pickle_slots__:
            result[i] = getattr(self, i)
        return result

    # Note: because NONE of the slots on this class need to be edited,
    # we don't need to implement a specialized __setstate__ method, and
    # can quietly rely on the super() class's implementation.
    # def __setstate__(self, state):
    #     pass

    def __call__(self, exception=True):
        """Return the value of this object."""
        return self.value

    def activate(self):
        self.active=True

    def deactivate(self):
        self.active=False

    def setlb(self, value):
        if value is None:
            self.lb = None
        else:
            if is_fixed(value):
                self.lb = value
            else:
                raise ValueError(
                    "Non-fixed input of type '%s' supplied as variable lower "
                    "bound - legal types must be fixed expressions or variables."
                    % (type(value),) )

    def setub(self, value):
        if value is None:
            self.ub = None
        else:
            if is_fixed(value):
                    self.ub = value
            else:
                raise ValueError(
                    "Non-fixed input of type '%s' supplied as variable upper "
                    "bound - legal types are fixed expressions or variables."
                    "parameters"
                    % (type(value),) )

    def is_fixed(self):
        if self.fixed:
            return True
        return False

    def is_constant(self):
        return False

    def polynomial_degree(self):
        if self.fixed:
            return 0
        return 1

    def is_binary(self):
        return self.component().is_binary(self.index())

    def is_integer(self):
        return self.component().is_integer(self.index())

    def is_continuous(self):
        return self.component().is_continuous(self.index())

    def pprint(self, ostream=None, verbose=False):
        if ostream is None:
            ostream = sys.stdout
        ostream.write(str(self))

    def set_value(self, val):
        """Set the value of this numeric object, after validating its value."""
        if self._valid_value(val):
            self.value=val

    def _valid_value(self, value, use_exception=True):
        """
        Validate the value.  If use_exception is True, then raise an
        exception.
        """
        ans = value is None or self.domain is None or value in self.domain
        if not ans and use_exception:
            raise ValueError("Numeric value `%s` is not in domain %s"
                             % (value, self.domain))
        return ans

#
# Variable attributes:
#
# Single variable:
#
# x = Var()
# x.fixed = True
# x.domain = Reals
# x.name = "x"
# x.setlb(-1.0)
# x.setub(1.0)
# x.initial = 0.0
# x = -0.4
#
# Array of variables:
#
# y = Var(set1,set2,...,setn)
# y.fixed = True (fixes all variables)
# y.domain = Reals
# y.name = "y"
# y[i,j,...,k] (returns value of a given index)
#
# y[i,j,...,k].fixed
# y[i,j,...,k].lb()
# y[i,j,...,k].ub()
# y[i,j,...,k].initial
#

class Var(IndexedComponent):
    """A numeric variable, which may be defined over a index"""

    """
    Constructor Arguments:
        name        The name of this variable
        index       The index set that defines the distinct variables.
                        By default, this is None, indicating that there
                        is a single variable.
        domain      A set that defines the type of values that
                        each variable must be.
        default     A set that defines default values for this
                        variable.
        bounds      A rule for defining bounds values for this
                        variable.
        rule        A rule for setting up this variable with
                        existing model data
    """

    def __new__(cls, *args, **kwds):
        if cls != Var:
            return super(Var, cls).__new__(cls)
        if args == ():
            return _VarElement.__new__(_VarElement)
        else:
            return _VarArray.__new__(_VarArray)

    def __init__(self, *args, **kwd):
        # Default keyword values
        self._initialize = kwd.pop('initialize', None )
        self.domain = kwd.pop('within', Reals )
        self.domain = kwd.pop('domain', self.domain )
        self.bounds = kwd.pop('bounds', None )

        self._defer_domain = False

        #Handle within=not_pyomo_object 
        if not getattr(self.domain, 'virtual', False):
            if self.domain == [] or self.domain == set([]):
                raise ValueError("Attempting to set a variable's domain to an empty set")
            else: 
                from coopr.pyomo.base.rangeset import RangeSet
                if type(self.domain) is RangeSet:
                    self.bounds = (self.domain._start,self.domain._end)
                    domain_name = self.domain.name
                    self.domain = IntegerSet()
                    self.domain.name = domain_name
                elif type(self.domain) is _SetContainer:
                    if self.domain.initialize is None:
                        self._defer_domain = True
                    elif self.domain.initialize == []:
                        raise ValueError("Attempting to set a variable's domain to an empty Set")
                    else:
                        self._defer_domain = False
                        domain_name = self.domain.name
                        self.domain = self.domain.initialize
                        self.domain.sort()
                        if len(self.domain) != self.domain[-1] - self.domain[0] + 1:
                            raise ValueError("Attempting to set a variable's domain to an improperly formatted Set " \
                                           + "-- elements are missing or duplicates exist: {0}".format(self.domain))
                        elif not all(type(elt) is int for elt in self.domain):
                            raise ValueError("Attempting to set a veriable's domain to a Set with noninteger elements: {0}".format(self.domain))
                        else:
                            self.bounds = (self.domain[0],self.domain[-1])
                            self.domain = IntegerSet()
                            self.domain.name = domain_name
                elif type(self.domain) is list:
                    self.domain.sort()
                    if len(self.domain) != self.domain[-1] - self.domain[0] + 1:
                        raise ValueError("Attempting to set a variable's domain to an improperly formatted set " \
                                       + "-- elements are missing or duplicates exist: {0}".format(self.domain))
                    elif not all(type(elt) is int for elt in self.domain):
                        raise ValueError("Attempting to set a veriable's domain to a set with noninteger elements: {0}".format(self.domain))
                    else:
                        self.bounds = (self.domain[0],self.domain[-1])
                        domain_name = "[{0}..{1}]".format(self.domain[0],self.domain[-1])
                        self.domain = IntegerSet()
                        self.domain.name = domain_name
                elif type(self.domain) is set:
                    self.domain = list(self.domain) 
                    self.domain.sort()
                    if len(self.domain) != self.domain[-1] - self.domain[0] + 1:
                        raise ValueError("Attempting to set a variable's domain to an improperly formatted set " \
                                       + "-- elements are missing or duplicates exist: {0}".format(self.domain))
                    elif not all(type(elt) is int for elt in self.domain):
                        raise ValueError("Attempting to set a veriable's domain to a set with noninteger elements: {0}".format(self.domain))
                    else:
                        self.bounds = (self.domain[0],self.domain[-1])
                        domain_name = "[{0}..{1}]".format(self.domain[0],self.domain[-1])
                        self.domain = IntegerSet()
                        self.domain.name = domain_name
                elif type(self.domain) is xrange:
                    self.bounds = (self.domain[0],self.domain[-1])
                    domain_name = "[{0}..{1}]".format(self.bounds[0],self.bounds[-1])
                    self.domain = IntegerSet()
                    self.domain.name = domain_name

        kwd.setdefault('ctype', Var)

        IndexedComponent.__init__(self, *args, **kwd)

        # check to see if domains for individual _VarData objects are specified via a rule.
        if isfunctor(self.domain):
            self._domain_rule = self.domain
            self.domain = None # there is no general domain for the collection of variables.
        else:
            self._domain_rule = None # there is no domain rule!

        # cached for efficiency purposes - isinstance is not cheap. 
        # only truly valid if there is no domain rule specified.
        if self.domain is not None:
            self._is_binary = isinstance(self.domain, BooleanSet)
            # second case is required in the case of deferred domains (the set may not be constructed yet, so hasn't been translated to an IntegerSet)
            self._is_integer = isinstance(self.domain, IntegerSet) or (type(self.domain) is _SetContainer)
            self._is_continuous = not (self._is_binary or self._is_integer)
        else:
            self._is_binary = None
            self._is_integer = None
            self._is_continuous = None

        # the following are only applicable if a domain rule *has* been specified.
        self._binary_keys = []
        self._integer_keys = []
        self._continuous_keys = []

    def as_numeric(self):
        if None in self._data:
            return self._data[None]
        return self

    def is_indexed(self):
        return self._ndim > 0

    def is_expression(self):
        return False

    def is_relational(self):
        return False

    def keys(self):
        return self._data.keys()

    # sets the "stale" attribute of every var index to True.
    def flag_as_stale(self):
        for var_data in itervalues(self._data):
            var_data.stale = True

    # returns a dictionary of index-value pairs.
    def extract_values(self):
        ans = {}
        for index, varval in iteritems(self._data):
            ans[index] = varval.value
        return ans

    # takes as input a dictionary of index-value pairs.
    def store_values(self, new_values):
        for index, varval in iteritems(self._data):
            varval.value = new_values[index]
            varval.stale = False

    def is_binary(self, index):
        if self._domain_rule is None:
            return self._is_binary
        else:
            return index in self._binary_keys

    def is_integer(self, index):
        if self._domain_rule is None:
            return self._is_integer
        else:
            return index in self._integer_keys

    def is_continuous(self, index):
        if self._domain_rule is None:
            return self._is_continuous
        else:
            return index in self._continuous_keys

    def binary_keys(self):
        """ Returns the keys of all binary variables """
        if self._domain_rule is None:
            if self._is_binary:
                return self._data.keys()
            else:
                return []
        else:
            return self._binary_keys

    def integer_keys(self):
        """ Returns the keys of all integer variables """
        if self._domain_rule is None:
            if self._is_integer:
                return self._data.keys()
            else:
                return []
        else:
            return self._integer_keys

    def continuous_keys(self):
        """ Returns the keys of all continuous variables """
        if self._domain_rule is None:
            if self._is_continuous:
                return self._data.keys()
            else:
                return []
        else:
            return self._continuous_keys

    def __iter__(self):
        return iterkeys(self._data)

    def iterkeys(self):
        return iterkeys(self._data)

    def itervalues(self):
        return itervalues(self._data)

    def iteritems(self):
        return iteritems(self._data)

    def __contains__(self, ndx):
        return ndx in self._data

    def dim(self):
        return self._ndim

    def reset(self):
        for value in itervalues(self._data):
            value.set_value(value.initial)

    def __len__(self):
        return len(self._data)

    def __setitem__(self,ndx,val):
        if (len(self._index) == 1) and (None in self._index) and (ndx is not None): # a somewhat gory way to see if you have a singleton - the len()=1 check is needed to avoid "None" being passed illegally into set membership validation rules.
            # allow for None indexing if this is truly a singleton
            msg = "Cannot set an array value in singleton variable '%s'"
            raise KeyError(msg % self.cname())

        if ndx not in self._index:
            msg = "Cannot set the value of array variable '%s' with invalid " \
                  "index '%s'"
            raise KeyError(msg % ( self.cname(), str(ndx) ))

        if not self._valid_value(val,False):
            msg = "Cannot set variable '%s[%s]' with invalid value: %s"
            raise ValueError(msg % ( self.cname(), str(ndx), str(val) ))
        self._data[ndx].value = val
        self._data[ndx].stale = False

    def __getitem__(self,ndx):
        """This method returns a _VarData object.  This object can be
           coerced to a numeric value using the value() function, or using
           explicity coercion with float().
        """
        try:
            return self._data[ndx]
        except KeyError: # thrown if the supplied index is hashable, but not defined.
            msg = "Unknown index '%s' in variable %s;" % (str(ndx), self.cname())
            if (isinstance(ndx, (tuple, list)) and len(ndx) != self.dim()):
                msg += "    Expecting %i-dimensional indices" % self.dim()
            else:
                msg += "    Make sure the correct index sets were used.\n"
                msg += "    Is the ordering of the indices correct?"
            raise KeyError(msg)
        except TypeError: # thrown if the supplied index is not hashable
            msg = sys.exc_info()[1]
            msg2 = "Unable to index variable %s using supplied index with " % self.cname()
            msg2 += str(msg)
            raise TypeError(msg2)

    def _add_indexed_member(self, ndx):
        # Check for domain rules
        if self._domain_rule is not None:
            domain = apply_indexed_rule( self, self._domain_rule,
                                         self._parent(), ndx )
            if isinstance(domain, BooleanSet):
                self._binary_keys.append(ndx)
            elif isinstance(domain, IntegerSet):
                self._integer_keys.append(ndx)
            else:
                self._continuous_keys.append(ndx)
        else:
            domain = self.domain

        self._data[ndx] = _VarData(self, domain)


    def construct(self, data=None):
        if __debug__:   #pragma:nocover
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug("Constructing Variable, name=%s, from data=%s", self.cname(), str(data))
        if self._constructed:
            return
        self._constructed=True
        #
        # Construct _VarData objects for all index values
        #
        if self._ndim > 0:

            for ndx in self._index:
                # Ignore a None index.  This occurs when an empty VarList is
                # created;  the empty implicit set has the None index.
                if ndx is not None: 
                    self._add_indexed_member(ndx)


        #
        # Define the _XXX_keys objects if domain isn't a rule;
        # they were defined individually above in the case of
        # a rule
        #
        if self._domain_rule is None:
            # **NOTE: In Py3k dict.keys() returns a dict_keys set-like object
            #         that can _NOT_ be pickled. 
            if isinstance(self.domain, BooleanSet):
                self._binary_keys = list(self._data.keys())
            elif isinstance(self.domain, IntegerSet):
                self._integer_keys = list(self._data.keys())
            else:
                self._continuous_keys = list(self._data.keys())

        #
        # Initialize values with a dictionary if provided
        #
        if self._initialize is not None:
            #
            # Initialize values with the _rule function if provided
            #
            if self._initialize.__class__ is types.FunctionType:
                for key in self._data:
                    if key is None:
                        val = self._initialize(self._parent())
                    else:
                        val = apply_indexed_rule( self, self._initialize,
                                                  self._parent(), key )
                    val = value(val)
                    self._valid_value(val, True)
                    self._data[key].value = self._data[key].initial = val
                    self._data[key].stale = False
            elif self._initialize.__class__ is dict:
                for key in self._initialize:
                    val = self._initialize[key]
                    self._valid_value(val, True)
                    self._data[key].value = self._data[key].initial = val
                    self._data[key].stale = False
            
            # BLN: The following is kind of a hack that John wrote
            # that allows you to initialize a variable with an object
            # which can be called. This is useful when you want to
            # initialize a variable using something like a map. It was
            # originally meant for the case of variables being indexed
            # by a differential set where you want to map any index in
            # the differential set to the appropriate finite
            # element. I dont currently use this anywhere but I
            # thought I'd commit it in case someone wanted to do
            # something similar in the future or make it less of a
            # hack. An example of the intended use is below:
            ########################
            #class mapToFE(object):
            #    def __init__(self, dataSource, diffSet, init=None, scale=1):
            #        self.dataSrc = dataSource
            #        self.diffSet = diffSet
            #        self.init = init
            #        self.scale = scale
            #    def __call__(self, model, point):
            #        if point == 0 and self.init is not None:
            #            ans = self.init
            #        else:
            #            ans = self.dataSrc[self.diffSet.get_upper_element_boundary(point)]
            #        return value(self.scale) * value(ans)
            #########################
            # elif hasattr(self._initialize, '__call__'):
            #     try:
            #         for key in self._data:
            #             if key is None:
            #                 val = self._initialize(self._parent())
            #             else:
            #                 val = apply_indexed_rule( self, self._initialize,
            #                                           self._parent(), key )
            #             val = value(val)
            #             self._valid_value(val, True)
            #             self._data[key].value = self._data[key].initial = val
            #             self._data[key].stale = False
            #     except:
            #         for key in self._data:
            #             val = self._initialize(self._parent())
            #             val = value(val)
            #             self._valid_value(val, True)
            #             self._data[key].value = self._data[key].initial = val
            #             self._data[key].stale = False
            else:
                for key in self._index:
                    val = self._initialize
                    self._valid_value(val, True)
                    self._data[key].value = self._data[key].initial = val
                    self._data[key].stale = False                    
        #
        # Initialize bounds with the bounds function if provided
        #
        if self.bounds is not None:

            if type(self.bounds) is tuple:

                # bounds are specified via a tuple - same lower and upper bounds for all var values!

                (lb, ub) = self.bounds

                # do some simple validation that the bounds are actually finite - otherwise, set them to None.
                if (lb is not None) and (not pyutilib.math.is_finite(value(lb))):
                    lb = None
                if (ub is not None) and (not pyutilib.math.is_finite(value(ub))):
                    ub = None

                for key,varval in iteritems(self._data):
                    if lb is not None:
                        varval.setlb(lb)
                    if ub is not None:
                        varval.setub(ub)

            else:

                # bounds are specified via a function

                for index, varval in iteritems(self._data):
                    if index is None:
                        (lb, ub) = self.bounds(self._parent())
                    else:
                        (lb, ub) = apply_indexed_rule( self, self.bounds,
                                                       self._parent(), index )
                    varval.setlb(lb)
                    varval.setub(ub)

                for varval in itervalues(self._data):
                    if varval.lb is not None and not pyutilib.math.is_finite(value(varval.lb)):
                        varval.setlb(None)
                    if varval.ub is not None and not pyutilib.math.is_finite(value(varval.ub)):
                        varval.setub(None)
        #
        # Iterate through all variables, and tighten the bounds based on
        # the domain bounds information.
        #
        # Only done if self.domain is not a rule. If it is, _VarArray level
        # bounds become meaningless, since the individual _VarElement objects
        # likely have more restricted domains.
        #
        if self._domain_rule is None:
            dbounds = self.domain.bounds()
            if not dbounds is None and dbounds != (None,None):
                for varval in itervalues(self._data):
                    if not dbounds[0] is None:
                        if varval.lb is None or                     \
                           dbounds[0] > value(varval.lb):
                            varval.setlb(dbounds[0])

                    if not dbounds[1] is None:
                        if varval.ub is None or                     \
                            dbounds[1] < value(varval.ub):
                            varval.setub(dbounds[1])

    def fix_all(self):
        for key in self:
            self[key].fixed = True

    def unfix_all(self):
        for key in self:
            self[key].fixed = False

    def pprint(self, ostream=None, verbose=False):
        if ostream is None:
            ostream = sys.stdout
        ostream.write("   "+self.cname()+" : ")
        if not self.doc is None:
            ostream.write(self.doc+'\n')
            ostream.write("  ")
        ostream.write("\tSize="+str(len(self))+" ")
        if self.domain is not None:
            ostream.write("\tDomain="+self.domain.name+"\n")
        else:
            ostream.write("\tDomain=None\n")
        if self._index_set is not None:
            ostream.write("\tIndicies:  ")
            for idx in self._index_set:
                ostream.write(str(idx.name)+",  ")
            ostream.write('\n')
        if None in self._data:
            ostream.write("\tInitial Value : Lower Bound : Upper Bound : "  \
                             "Current Value: Fixed: Stale\n")
            lb_value = None
            if self._data[None].lb is not None:
                lb_value = value(self._data[None].lb)
            ub_value = None
            if self._data[None].ub is not None:
                ub_value = value(self._data[None].ub)

            ostream.write("\t %s : %s : %s : %s : %s : %s\n" % (
              str(self._data[None].initial),
              str(lb_value),
              str(ub_value),
              str(self._data[None].value),
              str(self._data[None].fixed),
              str(self._data[None].stale)
            ))
        else:
            ostream.write("\tKey : Initial Value : Lower Bound : "          \
                             "Upper Bound : Current Value: Fixed: Stale\n")
            for key in sorted(self._data.keys()):
                initial_val = self._data[key].initial
                lb_value = None
                if self._data[key].lb is not None:
                    lb_value = value(self._data[key].lb)
                ub_value = None
                if self._data[key].ub is not None:
                    ub_value = value(self._data[key].ub)

                ostream.write("\t%s : %s : %s : %s : %s : %s : %s\n" % (
                  str(key),
                  str(initial_val),
                  str(lb_value),
                  str(ub_value),
                  str(value(self._data[key].value)),
                  str(value(self._data[key].fixed)),
                  str(self._data[key].stale)
                ))

    def display(self, prefix="", ostream=None):
        if ostream is None:
            ostream = sys.stdout
        ostream.write(prefix+"Variable "+self.cname()+" : ")
        ostream.write("Size="+str(len(self))+' ')
        ostream.write("Domain="+self.domain.name+'\n')
        if None in self._data:
            ostream.write("%s  Value=%s\n" % (
              prefix,
              pyutilib.misc.format_io(self._data[None].value)
            ))
        else:
            for key in self._data:
                val = self._data[key].value
                ostream.write(prefix+"  "+str(key)+" : "+str(val)+'\n')

# a _VarElement is the implementation representing a "singleton" or non-indexed variable.
# NOTE: this class derives from both a "slot"ized base class (_VarData) and a normal
#       class with a dictionary (Var) - beware (although I believe the __getstate__
#       implementation should be attribute-independent).

class _VarElement(Var, _VarData):

    def __init__(self, *args, **kwd):
        _VarData.__init__( self, self,
                           kwd.get('within', kwd.get('domain', Reals)) )
        Var.__init__(self, *args, **kwd)
        self._data[None] = self

    def __call__(self, exception=True):
        if None in self._data:
            return self._data[None].value
        return None

    # Since this class derives from Component and Component.__getstate__
    # just packs up the entire __dict__ into the state dict, there s
    # nothng special that we need to do here.  We will just defer to the
    # super() get/set state.  Since all of our get/set state methods
    # rely on super() to traverse the MRO, this will automatically pick
    # up both the Component and Data base classes.
    #
    #def __getstate__(self):
    #    pass
    #
    #def __setstate__(self, state):
    #    pass

    def is_constant(self):
        return _VarData.is_constant(self)

# a _VarArray is the implementation representing an indexed variable.

class _VarArray(Var):
    
    def __init__(self, *args, **kwds):
        Var.__init__(self, *args, **kwds)
        self._dummy_val = _VarData(
            self, kwds.get('within', kwds.get('domain', Reals)) )

    def __float__(self):
        raise TypeError("Cannot access the value of array variable "+self.cname())

    def __int__(self):
        raise TypeError("Cannot access the value of array variable "+self.cname())

    def _valid_value(self,value,use_exception=True):
        return self._dummy_val._valid_value(value, use_exception)

    def set_value(self, value):
        msg = "Cannot specify the value of array variable '%s'"
        raise ValueError(msg % self.cname())


class VarList(_VarArray):
    """
    Variable-length indexed variable objects used to construct Pyomo models.
    """

    End = ( 1003, )

    def __init__(self, *args, **kwargs):
        if len(args) > 0:
            raise ValueError("Cannot specify indices for a VarList object")
        # Construct parents...
        self._hidden_index = Set()
        _VarArray.__init__(self, self._hidden_index, **kwargs)
        self._ndim = 1
        self._nvars = 0

    def is_fixed(self):
        if self._nvars == 0:
            return False
        for idx in xrange(self._nvars):
            if not self[idx].is_fixed():
                return False
        return True

    def is_constant(self):
        return False

    def polynomial_degree(self):
        if self.is_fixed():
            return 0
        return 1

    def construct(self, data=None):
        if __debug__:
            logger.debug("Constructing variable list %s",self.cname())
        self._hidden_index.construct()
        _VarArray.construct(self, data)

    def add(self):
        self._hidden_index.add(self._nvars)
        self._add_indexed_member(self._nvars)
        self._nvars += 1
        return self[self._nvars-1]


register_component(Var, "Decision variables in a model.")
register_component(VarList, "Variable-length list of decision variables in a model.")

